@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -38,15 +38,17 @@
 #include <fcntl.h> /* for setmode() */
 #endif
 
+#define JNLIB_NEED_LOG_LOGV
 #include "agent.h"
+#include "minip12.h"
 #include "i18n.h"
 #include "get-passphrase.h"
 #include "sysutils.h"
-#include "../common/init.h"
+#include "estream.h"
 
 
-enum cmd_and_opt_values
-{
+enum cmd_and_opt_values 
+{ 
   aNull = 0,
   oVerbose	  = 'v',
   oArmor          = 'a',
@@ -54,7 +56,7 @@ enum cmd_and_opt_values
 
   oProtect        = 'p',
   oUnprotect      = 'u',
-
+  
   oNoVerbose = 500,
   oShadow,
   oShowShadowInfo,
@@ -62,20 +64,22 @@ enum cmd_and_opt_values
   oS2Kcalibration,
   oCanonical,
 
+  oP12Import,
+  oP12Export,
+  oP12Charset,
   oStore,
   oForce,
   oHaveCert,
   oNoFailOnExist,
   oHomedir,
   oPrompt,
-  oStatusMsg,
-  oDebugUseOCB,
+  oStatusMsg, 
 
   oAgentProgram
 };
 
 
-struct rsa_secret_key_s
+struct rsa_secret_key_s 
 {
   gcry_mpi_t n;	    /* public modulus */
   gcry_mpi_t e;	    /* public exponent */
@@ -86,6 +90,7 @@ struct rsa_secret_key_s
 };
 
 
+static const char *opt_homedir;
 static int opt_armor;
 static int opt_canonical;
 static int opt_store;
@@ -95,11 +100,13 @@ static int opt_have_cert;
 static const char *opt_passphrase;
 static char *opt_prompt;
 static int opt_status_msg;
-static const char *opt_agent_program;
-static int opt_debug_use_ocb;
+static const char *opt_p12_charset;
+static const char *opt_agent_program; 
 
 static char *get_passphrase (int promptno);
 static void release_passphrase (char *pw);
+static int store_private_key (const unsigned char *grip,
+                              const void *buffer, size_t length, int force);
 
 
 static ARGPARSE_OPTS opts[] = {
@@ -110,8 +117,13 @@ static ARGPARSE_OPTS opts[] = {
   ARGPARSE_c (oShadow,    "shadow", "create a shadow entry for a public key"),
   ARGPARSE_c (oShowShadowInfo,  "show-shadow-info", "return the shadow info"),
   ARGPARSE_c (oShowKeygrip, "show-keygrip", "show the \"keygrip\""),
-  ARGPARSE_c (oS2Kcalibration, "s2k-calibration", "@"),
+  ARGPARSE_c (oP12Import, "p12-import", 
+              "import a pkcs#12 encoded private key"),
+  ARGPARSE_c (oP12Export, "p12-export",
+              "export a private key pkcs#12 encoded"),
 
+  ARGPARSE_c (oS2Kcalibration, "s2k-calibration", "@"),
+  
   ARGPARSE_group (301, N_("@\nOptions:\n ")),
 
   ARGPARSE_s_n (oVerbose, "verbose", "verbose"),
@@ -119,22 +131,22 @@ static ARGPARSE_OPTS opts[] = {
   ARGPARSE_s_n (oCanonical, "canonical", "write output in canonical format"),
 
   ARGPARSE_s_s (oPassphrase, "passphrase", "|STRING|use passphrase STRING"),
+  ARGPARSE_s_s (oP12Charset,"p12-charset",
+                "|NAME|set charset for a new PKCS#12 passphrase to NAME"),
   ARGPARSE_s_n (oHaveCert, "have-cert",
                 "certificate to export provided on STDIN"),
-  ARGPARSE_s_n (oStore,    "store",
+  ARGPARSE_s_n (oStore,    "store", 
                 "store the created key in the appropriate place"),
-  ARGPARSE_s_n (oForce,    "force",
+  ARGPARSE_s_n (oForce,    "force", 
                 "force overwriting"),
   ARGPARSE_s_n (oNoFailOnExist, "no-fail-on-exist", "@"),
-  ARGPARSE_s_s (oHomedir, "homedir", "@"),
-  ARGPARSE_s_s (oPrompt,  "prompt",
-                "|ESCSTRING|use ESCSTRING as prompt in pinentry"),
+  ARGPARSE_s_s (oHomedir, "homedir", "@"), 
+  ARGPARSE_s_s (oPrompt,  "prompt", 
+                "|ESCSTRING|use ESCSTRING as prompt in pinentry"), 
   ARGPARSE_s_n (oStatusMsg, "enable-status-msg", "@"),
 
   ARGPARSE_s_s (oAgentProgram, "agent-program", "@"),
 
-  ARGPARSE_s_n (oDebugUseOCB,  "debug-use-ocb", "@"), /* For hacking only.  */
-
   ARGPARSE_end ()
 };
 
@@ -144,7 +156,7 @@ my_strusage (int level)
   const char *p;
   switch (level)
     {
-    case 11: p = "gpg-protect-tool (" GNUPG_NAME ")";
+    case 11: p = "gpg-protect-tool (GnuPG)";
       break;
     case 13: p = VERSION; break;
     case 17: p = PRINTABLE_OS_NAME; break;
@@ -156,7 +168,7 @@ my_strusage (int level)
     case 41: p =  _("Syntax: gpg-protect-tool [options] [args]\n"
                     "Secret key maintenance tool\n");
     break;
-
+    
     default: p = NULL;
     }
   return p;
@@ -193,7 +205,7 @@ make_canonical (const char *fname, const char *buf, size_t buflen)
   rc = gcry_sexp_sscan (&sexp, &erroff, buf, buflen);
   if (rc)
     {
-      log_error ("invalid S-Expression in '%s' (off=%u): %s\n",
+      log_error ("invalid S-Expression in `%s' (off=%u): %s\n",
                  fname, (unsigned int)erroff, gpg_strerror (rc));
       return NULL;
     }
@@ -237,7 +249,7 @@ read_file (const char *fname, size_t *r_length)
   FILE *fp;
   char *buf;
   size_t buflen;
-
+  
   if (!strcmp (fname, "-"))
     {
       size_t nread, bufsize = 0;
@@ -249,7 +261,7 @@ read_file (const char *fname, size_t *r_length)
       buf = NULL;
       buflen = 0;
 #define NCHUNK 8192
-      do
+      do 
         {
           bufsize += NCHUNK;
           if (!buf)
@@ -260,7 +272,7 @@ read_file (const char *fname, size_t *r_length)
           nread = fread (buf+buflen, 1, NCHUNK, fp);
           if (nread < NCHUNK && ferror (fp))
             {
-              log_error ("error reading '[stdin]': %s\n", strerror (errno));
+              log_error ("error reading `[stdin]': %s\n", strerror (errno));
               xfree (buf);
               return NULL;
             }
@@ -277,22 +289,22 @@ read_file (const char *fname, size_t *r_length)
       fp = fopen (fname, "rb");
       if (!fp)
         {
-          log_error ("can't open '%s': %s\n", fname, strerror (errno));
+          log_error ("can't open `%s': %s\n", fname, strerror (errno));
           return NULL;
         }
-
+  
       if (fstat (fileno(fp), &st))
         {
-          log_error ("can't stat '%s': %s\n", fname, strerror (errno));
+          log_error ("can't stat `%s': %s\n", fname, strerror (errno));
           fclose (fp);
           return NULL;
         }
-
+      
       buflen = st.st_size;
       buf = xmalloc (buflen+1);
       if (fread (buf, buflen, 1, fp) != 1)
         {
-          log_error ("error reading '%s': %s\n", fname, strerror (errno));
+          log_error ("error reading `%s': %s\n", fname, strerror (errno));
           fclose (fp);
           xfree (buf);
           return NULL;
@@ -311,7 +323,7 @@ read_key (const char *fname)
   char *buf;
   size_t buflen;
   unsigned char *key;
-
+  
   buf = read_file (fname, &buflen);
   if (!buf)
     return NULL;
@@ -330,14 +342,13 @@ read_and_protect (const char *fname)
   unsigned char *result;
   size_t resultlen;
   char *pw;
-
+  
   key = read_key (fname);
   if (!key)
     return;
 
   pw = get_passphrase (1);
-  rc = agent_protect (key, pw, &result, &resultlen, 0,
-                      opt_debug_use_ocb? 1 : -1);
+  rc = agent_protect (key, pw, &result, &resultlen);
   release_passphrase (pw);
   xfree (key);
   if (rc)
@@ -345,7 +356,7 @@ read_and_protect (const char *fname)
       log_error ("protecting the key failed: %s\n", gpg_strerror (rc));
       return;
     }
-
+  
   if (opt_armor)
     {
       char *p = make_advanced (result, resultlen);
@@ -362,7 +373,7 @@ read_and_protect (const char *fname)
 
 
 static void
-read_and_unprotect (ctrl_t ctrl, const char *fname)
+read_and_unprotect (const char *fname)
 {
   int  rc;
   unsigned char *key;
@@ -375,7 +386,7 @@ read_and_unprotect (ctrl_t ctrl, const char *fname)
   if (!key)
     return;
 
-  rc = agent_unprotect (ctrl, key, (pw=get_passphrase (1)),
+  rc = agent_unprotect (key, (pw=get_passphrase (1)), 
                         protected_at, &result, &resultlen);
   release_passphrase (pw);
   xfree (key);
@@ -387,14 +398,10 @@ read_and_unprotect (ctrl_t ctrl, const char *fname)
       return;
     }
   if (opt.verbose)
-    {
-      if (*protected_at)
-        log_info ("key protection done at %.4s-%.2s-%.2s %.2s:%.2s:%s\n",
-                  protected_at, protected_at+4, protected_at+6,
-                  protected_at+9, protected_at+11, protected_at+13);
-      else
-        log_info ("key protection done at [unknown]\n");
-    }
+    log_info ("key protection done at %.4s-%.2s-%.2s %.2s:%.2s:%s\n",
+              protected_at, protected_at+4, protected_at+6,
+              protected_at+9, protected_at+11, protected_at+13);
+
 
   if (opt_armor)
     {
@@ -420,7 +427,7 @@ read_and_shadow (const char *fname)
   unsigned char *result;
   size_t resultlen;
   unsigned char dummy_info[] = "(8:313233342:43)";
-
+  
   key = read_key (fname);
   if (!key)
     return;
@@ -434,7 +441,7 @@ read_and_shadow (const char *fname)
     }
   resultlen = gcry_sexp_canon_len (result, 0, NULL,NULL);
   assert (resultlen);
-
+  
   if (opt_armor)
     {
       char *p = make_advanced (result, resultlen);
@@ -456,7 +463,7 @@ show_shadow_info (const char *fname)
   unsigned char *key;
   const unsigned char *info;
   size_t infolen;
-
+  
   key = read_key (fname);
   if (!key)
     return;
@@ -470,7 +477,7 @@ show_shadow_info (const char *fname)
     }
   infolen = gcry_sexp_canon_len (info, 0, NULL,NULL);
   assert (infolen);
-
+  
   if (opt_armor)
     {
       char *p = make_advanced (info, infolen);
@@ -490,14 +497,14 @@ show_file (const char *fname)
   unsigned char *key;
   size_t keylen;
   char *p;
-
+  
   key = read_key (fname);
   if (!key)
     return;
 
   keylen = gcry_sexp_canon_len (key, 0, NULL,NULL);
   assert (keylen);
-
+  
   if (opt_canonical)
     {
       fwrite (key, keylen, 1, stdout);
@@ -521,7 +528,7 @@ show_keygrip (const char *fname)
   gcry_sexp_t private;
   unsigned char grip[20];
   int i;
-
+  
   key = read_key (fname);
   if (!key)
     return;
@@ -530,7 +537,7 @@ show_keygrip (const char *fname)
     {
       log_error ("gcry_sexp_new failed\n");
       return;
-    }
+    } 
   xfree (key);
 
   if (!gcry_pk_get_keygrip (private, grip))
@@ -546,6 +553,463 @@ show_keygrip (const char *fname)
 }
 
 
+static int
+rsa_key_check (struct rsa_secret_key_s *skey)
+{
+  int err = 0;
+  gcry_mpi_t t = gcry_mpi_snew (0);
+  gcry_mpi_t t1 = gcry_mpi_snew (0);
+  gcry_mpi_t t2 = gcry_mpi_snew (0);
+  gcry_mpi_t phi = gcry_mpi_snew (0);
+
+  /* check that n == p * q */
+  gcry_mpi_mul (t, skey->p, skey->q);
+  if (gcry_mpi_cmp( t, skey->n) )
+    {
+      log_error ("RSA oops: n != p * q\n");
+      err++;
+    }
+
+  /* check that p is less than q */
+  if (gcry_mpi_cmp (skey->p, skey->q) > 0)
+    {
+      gcry_mpi_t tmp;
+
+      log_info ("swapping secret primes\n");
+      tmp = gcry_mpi_copy (skey->p);
+      gcry_mpi_set (skey->p, skey->q);
+      gcry_mpi_set (skey->q, tmp);
+      gcry_mpi_release (tmp);
+      /* and must recompute u of course */
+      gcry_mpi_invm (skey->u, skey->p, skey->q);
+    }
+
+  /* check that e divides neither p-1 nor q-1 */
+  gcry_mpi_sub_ui (t, skey->p, 1 );
+  gcry_mpi_div (NULL, t, t, skey->e, 0);
+  if (!gcry_mpi_cmp_ui( t, 0) )
+    {
+      log_error ("RSA oops: e divides p-1\n");
+      err++;
+    }
+  gcry_mpi_sub_ui (t, skey->q, 1);
+  gcry_mpi_div (NULL, t, t, skey->e, 0);
+  if (!gcry_mpi_cmp_ui( t, 0))
+    {
+      log_info ( "RSA oops: e divides q-1\n" );
+      err++;
+    }
+
+  /* check that d is correct. */
+  gcry_mpi_sub_ui (t1, skey->p, 1);
+  gcry_mpi_sub_ui (t2, skey->q, 1);
+  gcry_mpi_mul (phi, t1, t2);
+  gcry_mpi_invm (t, skey->e, phi);
+  if (gcry_mpi_cmp (t, skey->d))
+    { /* no: try universal exponent. */
+      gcry_mpi_gcd (t, t1, t2);
+      gcry_mpi_div (t, NULL, phi, t, 0);
+      gcry_mpi_invm (t, skey->e, t);
+      if (gcry_mpi_cmp (t, skey->d))
+        {
+          log_error ("RSA oops: bad secret exponent\n");
+          err++;
+        }
+    }
+
+  /* check for correctness of u */
+  gcry_mpi_invm (t, skey->p, skey->q);
+  if (gcry_mpi_cmp (t, skey->u))
+    {
+      log_info ( "RSA oops: bad u parameter\n");
+      err++;
+    }
+
+  if (err)
+    log_info ("RSA secret key check failed\n");
+
+  gcry_mpi_release (t);
+  gcry_mpi_release (t1);
+  gcry_mpi_release (t2);
+  gcry_mpi_release (phi);
+
+  return err? -1:0;
+}
+
+
+/* A callback used by p12_parse to return a certificate.  */
+static void
+import_p12_cert_cb (void *opaque, const unsigned char *cert, size_t certlen)
+{
+  struct b64state state;
+  gpg_error_t err, err2;
+
+  (void)opaque;
+
+  err = b64enc_start (&state, stdout, "CERTIFICATE");
+  if (!err)
+    err = b64enc_write (&state, cert, certlen);
+  err2 = b64enc_finish (&state);
+  if (!err)
+    err = err2;
+  if (err)
+    log_error ("error writing armored certificate: %s\n", gpg_strerror (err));
+}
+
+static void
+import_p12_file (const char *fname)
+{
+  char *buf;
+  unsigned char *result;
+  size_t buflen, resultlen, buf_off;
+  int i;
+  int rc;
+  gcry_mpi_t *kparms;
+  struct rsa_secret_key_s sk;
+  gcry_sexp_t s_key;
+  unsigned char *key;
+  unsigned char grip[20];
+  char *pw;
+
+  /* fixme: we should release some stuff on error */
+  
+  buf = read_file (fname, &buflen);
+  if (!buf)
+    return;
+
+  /* GnuPG 2.0.4 accidently created binary P12 files with the string
+     "The passphrase is %s encoded.\n\n" prepended to the ASN.1 data.
+     We fix that here.  */
+  if (buflen > 29 && !memcmp (buf, "The passphrase is ", 18))
+    {
+      for (buf_off=18; buf_off < buflen && buf[buf_off] != '\n'; buf_off++)
+        ;
+      buf_off++;
+      if (buf_off < buflen && buf[buf_off] == '\n')
+        buf_off++;
+    }
+  else
+    buf_off = 0;
+
+  kparms = p12_parse ((unsigned char*)buf+buf_off, buflen-buf_off,
+                      (pw=get_passphrase (2)),
+                      import_p12_cert_cb, NULL);
+  release_passphrase (pw);
+  xfree (buf);
+  if (!kparms)
+    {
+      log_error ("error parsing or decrypting the PKCS-12 file\n");
+      return;
+    }
+  for (i=0; kparms[i]; i++)
+    ;
+  if (i != 8)
+    {
+      log_error ("invalid structure of private key\n");
+      return;
+    }
+
+
+/*    print_mpi ("   n", kparms[0]); */
+/*    print_mpi ("   e", kparms[1]); */
+/*    print_mpi ("   d", kparms[2]); */
+/*    print_mpi ("   p", kparms[3]); */
+/*    print_mpi ("   q", kparms[4]); */
+/*    print_mpi ("dmp1", kparms[5]); */
+/*    print_mpi ("dmq1", kparms[6]); */
+/*    print_mpi ("   u", kparms[7]); */
+
+  sk.n = kparms[0];
+  sk.e = kparms[1];
+  sk.d = kparms[2];
+  sk.q = kparms[3];
+  sk.p = kparms[4];
+  sk.u = kparms[7];
+  if (rsa_key_check (&sk))
+    return;
+/*    print_mpi ("   n", sk.n); */
+/*    print_mpi ("   e", sk.e); */
+/*    print_mpi ("   d", sk.d); */
+/*    print_mpi ("   p", sk.p); */
+/*    print_mpi ("   q", sk.q); */
+/*    print_mpi ("   u", sk.u); */
+
+  /* Create an S-expresion from the parameters. */
+  rc = gcry_sexp_build (&s_key, NULL,
+                        "(private-key(rsa(n%m)(e%m)(d%m)(p%m)(q%m)(u%m)))",
+                        sk.n, sk.e, sk.d, sk.p, sk.q, sk.u, NULL);
+  for (i=0; i < 8; i++)
+    gcry_mpi_release (kparms[i]);
+  gcry_free (kparms);
+  if (rc)
+    {
+      log_error ("failed to created S-expression from key: %s\n",
+                 gpg_strerror (rc));
+      return;
+    }
+
+  /* Compute the keygrip. */
+  if (!gcry_pk_get_keygrip (s_key, grip))
+    {
+      log_error ("can't calculate keygrip\n");
+      return;
+    }
+  log_info ("keygrip: ");
+  for (i=0; i < 20; i++)
+    log_printf ("%02X", grip[i]);
+  log_printf ("\n");
+
+  /* Convert to canonical encoding. */
+  buflen = gcry_sexp_sprint (s_key, GCRYSEXP_FMT_CANON, NULL, 0);
+  assert (buflen);
+  key = gcry_xmalloc_secure (buflen);
+  buflen = gcry_sexp_sprint (s_key, GCRYSEXP_FMT_CANON, key, buflen);
+  assert (buflen);
+  gcry_sexp_release (s_key);
+
+  pw = get_passphrase (4);
+  rc = agent_protect (key, pw, &result, &resultlen);
+  release_passphrase (pw);
+  xfree (key);
+  if (rc)
+    {
+      log_error ("protecting the key failed: %s\n", gpg_strerror (rc));
+      return;
+    }
+  
+  if (opt_armor)
+    {
+      char *p = make_advanced (result, resultlen);
+      xfree (result);
+      if (!p)
+        return;
+      result = (unsigned char*)p;
+      resultlen = strlen (p);
+    }
+
+  if (opt_store)
+    store_private_key (grip, result, resultlen, opt_force);
+  else
+    fwrite (result, resultlen, 1, stdout);
+
+  xfree (result);
+}
+
+
+
+static gcry_mpi_t *
+sexp_to_kparms (gcry_sexp_t sexp)
+{
+  gcry_sexp_t list, l2;
+  const char *name;
+  const char *s;
+  size_t n;
+  int i, idx;
+  const char *elems;
+  gcry_mpi_t *array;
+
+  list = gcry_sexp_find_token (sexp, "private-key", 0 );
+  if(!list)
+    return NULL; 
+  l2 = gcry_sexp_cadr (list);
+  gcry_sexp_release (list);
+  list = l2;
+  name = gcry_sexp_nth_data (list, 0, &n);
+  if(!name || n != 3 || memcmp (name, "rsa", 3))
+    {
+      gcry_sexp_release (list);
+      return NULL;
+    }
+
+  /* Parameter names used with RSA. */
+  elems = "nedpqu";
+  array = xcalloc (strlen(elems) + 1, sizeof *array);
+  for (idx=0, s=elems; *s; s++, idx++ ) 
+    {
+      l2 = gcry_sexp_find_token (list, s, 1);
+      if (!l2)
+        {
+          for (i=0; i<idx; i++)
+            gcry_mpi_release (array[i]);
+          xfree (array);
+          gcry_sexp_release (list);
+          return NULL; /* required parameter not found */
+	}
+      array[idx] = gcry_sexp_nth_mpi (l2, 1, GCRYMPI_FMT_USG);
+      gcry_sexp_release (l2);
+      if (!array[idx])
+        {
+          for (i=0; i<idx; i++)
+            gcry_mpi_release (array[i]);
+          xfree (array);
+          gcry_sexp_release (list);
+          return NULL; /* required parameter is invalid */
+	}
+    }
+  
+  gcry_sexp_release (list);
+  return array;
+}
+
+
+/* Check whether STRING is a KEYGRIP, i.e has the correct length and
+   does only consist of uppercase hex characters. */
+static int
+is_keygrip (const char *string)
+{
+  int i;
+
+  for(i=0; string[i] && i < 41; i++) 
+    if (!strchr("01234567890ABCDEF", string[i]))
+      return 0; 
+  return i == 40;
+}
+
+
+static void
+export_p12_file (const char *fname)
+{
+  int rc;
+  gcry_mpi_t kparms[9], *kp;
+  unsigned char *key;
+  size_t keylen;
+  gcry_sexp_t private;
+  struct rsa_secret_key_s sk;
+  int i;
+  unsigned char *cert = NULL;
+  size_t certlen = 0;
+  int keytype;
+  size_t keylen_for_wipe = 0;
+  char *pw;
+
+  if ( is_keygrip (fname) )
+    {
+      char hexgrip[40+4+1];
+      char *p;
+  
+      assert (strlen(fname) == 40);
+      strcpy (stpcpy (hexgrip, fname), ".key");
+
+      p = make_filename (opt_homedir, GNUPG_PRIVATE_KEYS_DIR, hexgrip, NULL);
+      key = read_key (p);
+      xfree (p);
+    }
+  else
+    key = read_key (fname);
+
+  if (!key)
+    return;
+
+  keytype = agent_private_key_type (key);
+  if (keytype == PRIVATE_KEY_PROTECTED)
+    {
+      unsigned char *tmpkey;
+      size_t tmplen;
+
+      rc = agent_unprotect (key, (pw=get_passphrase (1)),
+                            NULL, &tmpkey, &tmplen);
+      release_passphrase (pw);
+      if (rc)
+        {
+          if (opt_status_msg && gpg_err_code (rc) == GPG_ERR_BAD_PASSPHRASE )
+            log_info ("[PROTECT-TOOL:] bad-passphrase\n");
+          log_error ("unprotecting key `%s' failed: %s\n",
+                     fname, gpg_strerror (rc));
+          xfree (key);
+          return;
+        }
+      xfree (key);
+      key = tmpkey;
+      keylen_for_wipe = tmplen;
+
+      keytype = agent_private_key_type (key);
+    }
+
+  if (keytype == PRIVATE_KEY_SHADOWED)
+    {
+      log_error ("`%s' is a shadowed private key - can't export it\n", fname);
+      wipememory (key, keylen_for_wipe);
+      xfree (key);
+      return;
+    }
+  else if (keytype != PRIVATE_KEY_CLEAR)
+    {
+      log_error ("\%s' is not a private key\n", fname);
+      wipememory (key, keylen_for_wipe);
+      xfree (key);
+      return;
+    }
+
+
+  if (opt_have_cert)
+    {
+      cert = (unsigned char*)read_file ("-", &certlen);
+      if (!cert)
+        {
+          wipememory (key, keylen_for_wipe);
+          xfree (key);
+          return;
+        }
+    }
+
+
+  if (gcry_sexp_new (&private, key, 0, 0))
+    {
+      log_error ("gcry_sexp_new failed\n");
+      wipememory (key, keylen_for_wipe);
+      xfree (key);
+      xfree (cert);
+      return;
+    } 
+  wipememory (key, keylen_for_wipe);
+  xfree (key);
+
+  kp = sexp_to_kparms (private);
+  gcry_sexp_release (private);
+  if (!kp)
+    {
+      log_error ("error converting key parameters\n");
+      xfree (cert);
+      return;
+    } 
+  sk.n = kp[0];
+  sk.e = kp[1];
+  sk.d = kp[2];
+  sk.p = kp[3];
+  sk.q = kp[4];
+  sk.u = kp[5];
+  xfree (kp);
+
+ 
+  kparms[0] = sk.n;
+  kparms[1] = sk.e;
+  kparms[2] = sk.d;
+  kparms[3] = sk.q;
+  kparms[4] = sk.p;
+  kparms[5] = gcry_mpi_snew (0);  /* compute d mod (p-1) */
+  gcry_mpi_sub_ui (kparms[5], kparms[3], 1);
+  gcry_mpi_mod (kparms[5], sk.d, kparms[5]);   
+  kparms[6] = gcry_mpi_snew (0);  /* compute d mod (q-1) */
+  gcry_mpi_sub_ui (kparms[6], kparms[4], 1);
+  gcry_mpi_mod (kparms[6], sk.d, kparms[6]);   
+  kparms[7] = sk.u;
+  kparms[8] = NULL;
+
+  pw = get_passphrase (3);
+  key = p12_build (kparms, cert, certlen, pw, opt_p12_charset, &keylen);
+  release_passphrase (pw);
+  xfree (cert);
+  for (i=0; i < 8; i++)
+    gcry_mpi_release (kparms[i]);
+  if (!key)
+    return;
+  
+#ifdef HAVE_DOSISH_SYSTEM
+  setmode ( fileno (stdout) , O_BINARY );
+#endif
+  fwrite (key, keylen, 1, stdout);
+  xfree (key);
+}
 
 
 
@@ -555,20 +1019,28 @@ main (int argc, char **argv )
   ARGPARSE_ARGS pargs;
   int cmd = 0;
   const char *fname;
-  ctrl_t ctrl;
 
-  early_system_init ();
   set_strusage (my_strusage);
   gcry_control (GCRYCTL_SUSPEND_SECMEM_WARN);
-  log_set_prefix ("gpg-protect-tool", GPGRT_LOG_WITH_PREFIX);
+  log_set_prefix ("gpg-protect-tool", 1); 
 
   /* Make sure that our subsystems are ready.  */
   i18n_init ();
-  init_common_subsystems (&argc, &argv);
+  init_common_subsystems ();
+
+  if (!gcry_check_version (NEED_LIBGCRYPT_VERSION) )
+    {
+      log_fatal( _("%s is too old (need %s, have %s)\n"), "libgcrypt",
+                 NEED_LIBGCRYPT_VERSION, gcry_check_version (NULL) );
+    }
 
   setup_libgcrypt_logging ();
   gcry_control (GCRYCTL_INIT_SECMEM, 16384, 0);
 
+
+  opt_homedir = default_homedir ();
+
+
   pargs.argc = &argc;
   pargs.argv = &argv;
   pargs.flags=  1;  /* (do not remove the args) */
@@ -579,7 +1051,7 @@ main (int argc, char **argv )
         case oVerbose: opt.verbose++; break;
         case oArmor:   opt_armor=1; break;
         case oCanonical: opt_canonical=1; break;
-        case oHomedir: gnupg_set_homedir (pargs.r.ret_str); break;
+        case oHomedir: opt_homedir = pargs.r.ret_str; break;
 
         case oAgentProgram: opt_agent_program = pargs.r.ret_str; break;
 
@@ -588,6 +1060,10 @@ main (int argc, char **argv )
         case oShadow: cmd = oShadow; break;
         case oShowShadowInfo: cmd = oShowShadowInfo; break;
         case oShowKeygrip: cmd = oShowKeygrip; break;
+        case oP12Import: cmd = oP12Import; break;
+        case oP12Export: cmd = oP12Export; break;
+        case oP12Charset: opt_p12_charset = pargs.r.ret_str; break;
+
         case oS2Kcalibration: cmd = oS2Kcalibration; break;
 
         case oPassphrase: opt_passphrase = pargs.r.ret_str; break;
@@ -597,8 +1073,7 @@ main (int argc, char **argv )
         case oHaveCert: opt_have_cert = 1; break;
         case oPrompt: opt_prompt = pargs.r.ret_str; break;
         case oStatusMsg: opt_status_msg = 1; break;
-        case oDebugUseOCB: opt_debug_use_ocb = 1; break;
-
+          
         default: pargs.err = ARGPARSE_PRINT_ERROR; break;
 	}
     }
@@ -611,18 +1086,10 @@ main (int argc, char **argv )
   else if (argc > 1)
     usage (1);
 
-  /* Allocate an CTRL object.  An empty object should be sufficient.  */
-  ctrl = xtrycalloc (1, sizeof *ctrl);
-  if (!ctrl)
-    {
-      log_error ("error allocating connection control data: %s\n",
-                 strerror (errno));
-      agent_exit (1);
-    }
-
   /* Set the information which can't be taken from envvars.  */
   gnupg_prepare_get_passphrase (GPG_ERR_SOURCE_DEFAULT,
                                 opt.verbose,
+                                opt_homedir,
                                 opt_agent_program,
                                 NULL, NULL, NULL);
 
@@ -632,13 +1099,17 @@ main (int argc, char **argv )
   if (cmd == oProtect)
     read_and_protect (fname);
   else if (cmd == oUnprotect)
-    read_and_unprotect (ctrl, fname);
+    read_and_unprotect (fname);
   else if (cmd == oShadow)
     read_and_shadow (fname);
   else if (cmd == oShowShadowInfo)
     show_shadow_info (fname);
   else if (cmd == oShowKeygrip)
     show_keygrip (fname);
+  else if (cmd == oP12Import)
+    import_p12_file (fname);
+  else if (cmd == oP12Export)
+    export_p12_file (fname);
   else if (cmd == oS2Kcalibration)
     {
       if (!opt.verbose)
@@ -648,8 +1119,6 @@ main (int argc, char **argv )
   else
     show_file (fname);
 
-  xfree (ctrl);
-
   agent_exit (0);
   return 8; /*NOTREACHED*/
 }
@@ -678,7 +1147,7 @@ get_passphrase (int promptno)
   const char *desc;
   char *orig_codeset;
   int repeat = 0;
-
+  
   if (opt_passphrase)
     return xstrdup (opt_passphrase);
 
@@ -715,8 +1184,7 @@ get_passphrase (int promptno)
                               repeat, repeat, 1, &pw);
   if (err)
     {
-      if (gpg_err_code (err) == GPG_ERR_CANCELED
-          || gpg_err_code (err) == GPG_ERR_FULLY_CANCELED)
+      if (gpg_err_code (err) == GPG_ERR_CANCELED)
         log_info (_("cancelled\n"));
       else
         log_error (_("error while asking for the passphrase: %s\n"),
@@ -739,81 +1207,65 @@ release_passphrase (char *pw)
     }
 }
 
-
-/* Stub function.  */
-int
-agent_key_available (const unsigned char *grip)
-{
-  (void)grip;
-  return -1;  /* Not available.  */
-}
-
-char *
-agent_get_cache (const char *key, cache_mode_t cache_mode)
-{
-  (void)key;
-  (void)cache_mode;
-  return NULL;
-}
-
-gpg_error_t
-agent_askpin (ctrl_t ctrl,
-              const char *desc_text, const char *prompt_text,
-              const char *initial_errtext,
-              struct pin_entry_info_s *pininfo,
-              const char *keyinfo, cache_mode_t cache_mode)
-{
-  gpg_error_t err;
-  unsigned char *passphrase;
-  size_t size;
-
-  (void)ctrl;
-  (void)desc_text;
-  (void)prompt_text;
-  (void)initial_errtext;
-  (void)keyinfo;
-  (void)cache_mode;
-
-  *pininfo->pin = 0; /* Reset the PIN. */
-  passphrase = get_passphrase (0);
-  size = strlen (passphrase);
-  if (size >= pininfo->max_length)
-    return gpg_error (GPG_ERR_TOO_LARGE);
-
-  memcpy (&pininfo->pin, passphrase, size);
-  xfree (passphrase);
-  pininfo->pin[size] = 0;
-  if (pininfo->check_cb)
-    {
-      /* More checks by utilizing the optional callback. */
-      pininfo->cb_errtext = NULL;
-      err = pininfo->check_cb (pininfo);
-    }
-  else
-    err = 0;
-  return err;
-}
-
-/* Replacement for the function in findkey.c.  Here we write the key
- * to stdout. */
-int
-agent_write_private_key (const unsigned char *grip,
-                         const void *buffer, size_t length, int force)
+static int
+store_private_key (const unsigned char *grip,
+                   const void *buffer, size_t length, int force)
 {
+  char *fname;
+  estream_t fp;
   char hexgrip[40+4+1];
-  char *p;
-
-  (void)force;
-
+  
   bin2hex (grip, 20, hexgrip);
   strcpy (hexgrip+40, ".key");
-  p = make_advanced (buffer, length);
-  if (p)
+
+  fname = make_filename (opt_homedir, GNUPG_PRIVATE_KEYS_DIR, hexgrip, NULL);
+  if (force)
+    fp = es_fopen (fname, "wb");
+  else
     {
-      printf ("# Begin dump of %s\n%s%s# End dump of %s\n",
-              hexgrip, p, (*p && p[strlen(p)-1] == '\n')? "":"\n", hexgrip);
-      xfree (p);
+      if (!access (fname, F_OK))
+      {
+        if (opt_status_msg)
+          log_info ("[PROTECT-TOOL:] secretkey-exists\n");
+        if (opt_no_fail_on_exist)
+          log_info ("secret key file `%s' already exists\n", fname);
+        else
+          log_error ("secret key file `%s' already exists\n", fname);
+        xfree (fname);
+        return opt_no_fail_on_exist? 0 : -1;
+      }
+      /* FWIW: Under Windows Vista the standard fopen in the msvcrt
+         fails if the "x" GNU extension is used.  */
+      fp = es_fopen (fname, "wbx"); 
     }
 
+  if (!fp) 
+    { 
+      log_error ("can't create `%s': %s\n", fname, strerror (errno));
+      xfree (fname);
+      return -1;
+    }
+
+  if (es_fwrite (buffer, length, 1, fp) != 1)
+    {
+      log_error ("error writing `%s': %s\n", fname, strerror (errno));
+      es_fclose (fp);
+      remove (fname);
+      xfree (fname);
+      return -1;
+    }
+  if (es_fclose (fp))
+    {
+      log_error ("error closing `%s': %s\n", fname, strerror (errno));
+      remove (fname);
+      xfree (fname);
+      return -1;
+    }
+  log_info ("secret key stored as `%s'\n", fname);
+
+  if (opt_status_msg)
+    log_info ("[PROTECT-TOOL:] secretkey-stored\n");
+
+  xfree (fname);
   return 0;
 }
