@@ -3,22 +3,12 @@
  *
  * This file is part of GnuPG.
  *
- * This file is free software; you can redistribute it and/or modify
- * it under the terms of either
+ * GnuPG is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
  *
- *   - the GNU Lesser General Public License as published by the Free
- *     Software Foundation; either version 3 of the License, or (at
- *     your option) any later version.
- *
- * or
- *
- *   - the GNU General Public License as published by the Free
- *     Software Foundation; either version 2 of the License, or (at
- *     your option) any later version.
- *
- * or both in parallel, as here.
- *
- * This file is distributed in the hope that it will be useful,
+ * GnuPG is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -32,14 +22,16 @@
 #include <errno.h>
 
 #include "util.h"
+#include "iobuf.h"
+#include "estream-printf.h"
+
+#if !defined(_ESTREAM_PRINTF_MALLOC) || !defined(_ESTREAM_PRINTF_FREE)
+#error Need to define ESTREAM_PRINTF_MALLOC and _FREE
+#endif
 
 /* Same as asprintf but return an allocated buffer suitable to be
    freed using xfree.  This function simply dies on memory failure,
-   thus no extra check is required.
-
-   FIXME: We should remove these functions in favor of gpgrt_bsprintf
-   and a xgpgrt_bsprintf or rename them to xbsprintf and
-   xtrybsprintf.  */
+   thus no extra check is required. */
 char *
 xasprintf (const char *fmt, ...)
 {
@@ -47,7 +39,7 @@ xasprintf (const char *fmt, ...)
   char *buf;
 
   va_start (ap, fmt);
-  if (gpgrt_vasprintf (&buf, fmt, ap) < 0)
+  if (estream_vasprintf (&buf, fmt, ap) < 0)
     log_fatal ("estream_asprintf failed: %s\n", strerror (errno));
   va_end (ap);
   return buf;
@@ -62,7 +54,7 @@ xtryasprintf (const char *fmt, ...)
   char *buf;
 
   va_start (ap, fmt);
-  rc = gpgrt_vasprintf (&buf, fmt, ap);
+  rc = estream_vasprintf (&buf, fmt, ap);
   va_end (ap);
   if (rc < 0)
     return NULL;
