@@ -1,6 +1,6 @@
 /* compress.c - compress filter
  * Copyright (C) 1998, 1999, 2000, 2001, 2002,
- *               2003, 2006, 2010 Free Software Foundation, Inc.
+ *               2003, 2006 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 /* Note that the code in compress-bz2.c is nearly identical to the
@@ -29,13 +29,12 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <assert.h>
 #include <errno.h>
-#ifdef HAVE_ZIP
-# include <zlib.h>
-# if defined(__riscos__) && defined(USE_ZLIBRISCOS)
-#  include "zlib-riscos.h"
-# endif
-#endif
+#include <zlib.h>
+#if defined(__riscos__) && defined(USE_ZLIBRISCOS)
+# include "zlib-riscos.h"
+#endif 
 
 #include "gpg.h"
 #include "util.h"
@@ -47,7 +46,7 @@
 
 #ifdef __riscos__
 #define BYTEF_CAST(a) ((Bytef *)(a))
-#else
+#else 
 #define BYTEF_CAST(a) (a)
 #endif
 
@@ -56,7 +55,6 @@
 int compress_filter_bz2( void *opaque, int control,
 			 IOBUF a, byte *buf, size_t *ret_len);
 
-#ifdef HAVE_ZIP
 static void
 init_compress( compress_filter_context_t *zfx, z_stream *zs )
 {
@@ -139,7 +137,7 @@ init_uncompress( compress_filter_context_t *zfx, z_stream *zs )
      * PGP uses a windowsize of 13 bits. Using a negative value for
      * it forces zlib not to expect a zlib header.  This is a
      * undocumented feature Peter Gutmann told me about.
-     *
+     *    
      * We must use 15 bits for the inflator because CryptoEx uses 15
      * bits thus the output would get scrambled w/o error indication
      * if we would use 13 bits.  For the uncompressing this does not
@@ -163,8 +161,7 @@ do_uncompress( compress_filter_context_t *zfx, z_stream *zs,
 	       IOBUF a, size_t *ret_len )
 {
     int zrc;
-    int rc = 0;
-    int leave = 0;
+    int rc=0;
     size_t n;
     int nread, count;
     int refill = !zs->avail_in;
@@ -182,14 +179,13 @@ do_uncompress( compress_filter_context_t *zfx, z_stream *zs,
 	    nread = iobuf_read( a, zfx->inbuf + n, count );
 	    if( nread == -1 ) nread = 0;
 	    n += nread;
-	    /* Algo 1 has no zlib header which requires us to give
-	     * inflate an extra dummy byte to read. To be on the safe
-	     * side we allow for up to 4 ff bytes.  */
-	    if( nread < count && zfx->algo == 1 && zfx->algo1hack < 4) {
-		*(zfx->inbuf + n) = 0xFF;
-		zfx->algo1hack++;
+	    /* If we use the undocumented feature to suppress
+	     * the zlib header, we have to give inflate an
+	     * extra dummy byte to read */
+	    if( nread < count && zfx->algo == 1 ) {
+		*(zfx->inbuf + n) = 0xFF; /* is it really needed ? */
+		zfx->algo1hack = 1;
 		n++;
-                leave = 1;
 	    }
 	    zs->avail_in = n;
 	}
@@ -209,8 +205,7 @@ do_uncompress( compress_filter_context_t *zfx, z_stream *zs,
 	    else
 		log_fatal("zlib inflate problem: rc=%d\n", zrc );
 	}
-    } while (zs->avail_out && zrc != Z_STREAM_END && zrc != Z_BUF_ERROR
-             && !leave);
+    } while( zs->avail_out && zrc != Z_STREAM_END && zrc != Z_BUF_ERROR );
 
     *ret_len = zfx->outbufsize - zs->avail_out;
     if( DBG_FILTER )
@@ -250,9 +245,6 @@ compress_filter( void *opaque, int control,
 	    memset( &cd, 0, sizeof cd );
 	    cd.len = 0;
 	    cd.algorithm = zfx->algo;
-            /* Fixme: We should force a new CTB here:
-               cd.new_ctb = zfx->new_ctb;
-            */
 	    init_packet( &pkt );
 	    pkt.pkttype = PKT_COMPRESSED;
 	    pkt.pkt.compressed = &cd;
@@ -287,18 +279,14 @@ compress_filter( void *opaque, int control,
           zfx->release (zfx);
     }
     else if( control == IOBUFCTRL_DESC )
-        mem2str (buf, "compress_filter", *ret_len);
+	*(char**)buf = "compress_filter";
     return rc;
 }
-#endif /*HAVE_ZIP*/
+
 
 static void
 release_context (compress_filter_context_t *ctx)
 {
-  xfree(ctx->inbuf);
-  ctx->inbuf = NULL;
-  xfree(ctx->outbuf);
-  ctx->outbuf = NULL;
   xfree (ctx);
 }
 
@@ -306,14 +294,14 @@ release_context (compress_filter_context_t *ctx)
  * Handle a compressed packet
  */
 int
-handle_compressed (ctrl_t ctrl, void *procctx, PKT_compressed *cd,
+handle_compressed( void *procctx, PKT_compressed *cd,
 		   int (*callback)(IOBUF, void *), void *passthru )
 {
     compress_filter_context_t *cfx;
     int rc;
 
     if(check_compress_algo(cd->algorithm))
-      return GPG_ERR_COMPR_ALGO;
+      return G10ERR_COMPR_ALGO;
     cfx = xmalloc_clear (sizeof *cfx);
     cfx->release = release_context;
     cfx->algo = cd->algorithm;
@@ -321,7 +309,7 @@ handle_compressed (ctrl_t ctrl, void *procctx, PKT_compressed *cd,
     if( callback )
 	rc = callback(cd->buf, passthru );
     else
-      rc = proc_packets (ctrl,procctx, cd->buf);
+	rc = proc_packets(procctx, cd->buf);
     cd->buf = NULL;
     return rc;
 }
@@ -346,12 +334,10 @@ push_compress_filter2(IOBUF out,compress_filter_context_t *zfx,
     case COMPRESS_ALGO_NONE:
       break;
 
-#ifdef HAVE_ZIP
     case COMPRESS_ALGO_ZIP:
     case COMPRESS_ALGO_ZLIB:
       iobuf_push_filter2(out,compress_filter,zfx,rel);
       break;
-#endif
 
 #ifdef HAVE_BZIP2
     case COMPRESS_ALGO_BZIP2:
