@@ -32,11 +32,11 @@
 #endif
 
 #include "gpg.h"
-#include "util.h"
 #include "packet.h"
 #include "status.h"
 #include "exec.h"
 #include "keydb.h"
+#include "util.h"
 #include "i18n.h"
 #include "iobuf.h"
 #include "options.h"
@@ -48,7 +48,7 @@
 /* Generate a new photo id packet, or return NULL if canceled.
    FIXME:  Should we add a duplicates check similar to generate_user_id? */
 PKT_user_id *
-generate_photo_id (ctrl_t ctrl, PKT_public_key *pk,const char *photo_name)
+generate_photo_id(PKT_public_key *pk,const char *photo_name)
 {
   PKT_user_id *uid;
   int error=1,i;
@@ -113,11 +113,11 @@ generate_photo_id (ctrl_t ctrl, PKT_public_key *pk,const char *photo_name)
         {
           iobuf_close (file);
           file = NULL;
-          gpg_err_set_errno (EPERM);
+          errno = EPERM;
         }
       if(!file)
 	{
-	  log_error(_("unable to open JPEG file '%s': %s\n"),
+	  log_error(_("unable to open JPEG file `%s': %s\n"),
 		    filename,strerror(errno));
 	  xfree(filename);
 	  filename=NULL;
@@ -146,7 +146,7 @@ generate_photo_id (ctrl_t ctrl, PKT_public_key *pk,const char *photo_name)
       /* Is it a JPEG? */
       if(photo[0]!=0xFF || photo[1]!=0xD8)
 	{
-	  log_error(_("'%s' is not a JPEG file\n"),filename);
+	  log_error(_("`%s' is not a JPEG file\n"),filename);
 	  xfree(photo);
 	  photo=NULL;
 	  xfree(filename);
@@ -163,7 +163,7 @@ generate_photo_id (ctrl_t ctrl, PKT_public_key *pk,const char *photo_name)
          "user" may not be able to dismiss a viewer window! */
       if(opt.command_fd==-1)
 	{
-	  show_photos (ctrl, uid->attribs, uid->numattribs, pk, uid);
+	  show_photos(uid->attribs,uid->numattribs,pk,NULL,uid);
 	  switch(cpr_get_answer_yes_no_quit("photoid.jpeg.okay",
 					 _("Is this photo correct (y/N/q)? ")))
 	    {
@@ -286,31 +286,27 @@ static const char *get_default_photo_command(void)
 }
 #endif
 
-
 void
-show_photos (ctrl_t ctrl, const struct user_attribute *attrs, int count,
-             PKT_public_key *pk, PKT_user_id *uid)
+show_photos(const struct user_attribute *attrs,
+	    int count,PKT_public_key *pk,PKT_secret_key *sk,
+	    PKT_user_id *uid)
 {
-#ifdef DISABLE_PHOTO_VIEWER
-  (void)attrs;
-  (void)count;
-  (void)pk;
-  (void)uid;
-#else /*!DISABLE_PHOTO_VIEWER*/
+#ifndef DISABLE_PHOTO_VIEWER
   int i;
   struct expando_args args;
   u32 len;
   u32 kid[2]={0,0};
 
-  memset (&args, 0, sizeof(args));
-  args.pk = pk;
-  args.validity_info = get_validity_info (ctrl, pk, uid);
-  args.validity_string = get_validity_string (ctrl, pk, uid);
-  namehash_from_uid (uid);
-  args.namehash = uid->namehash;
+  memset(&args,0,sizeof(args));
+  args.pk=pk;
+  args.sk=sk;
+  args.validity_info=get_validity_info(pk,uid);
+  args.validity_string=get_validity_string(pk,uid);
 
-  if (pk)
-    keyid_from_pk (pk, kid);
+  if(pk)
+    keyid_from_pk(pk,kid);
+  else if(sk)
+    keyid_from_sk(sk,kid);
 
   for(i=0;i<count;i++)
     if(attrs[i].type==ATTRIB_IMAGE &&
@@ -378,5 +374,5 @@ show_photos (ctrl_t ctrl, const struct user_attribute *attrs, int count,
 
  fail:
   log_error(_("unable to display photo ID!\n"));
-#endif /*!DISABLE_PHOTO_VIEWER*/
+#endif
 }
