@@ -19,6 +19,7 @@
 
 #include <config.h>
 #include <stdio.h>
+#include <assert.h>
 
 #include "gpg.h"
 #include "iobuf.h"
@@ -63,7 +64,7 @@ release_progress_context (progress_filter_context_t *pfx)
 {
   if (!pfx)
     return;
-  log_assert (pfx->refcount);
+  assert (pfx->refcount);
   if ( --pfx->refcount )
     return;
   xfree (pfx->what);
@@ -71,63 +72,6 @@ release_progress_context (progress_filter_context_t *pfx)
 }
 
 
-static void
-write_status_progress (const char *what,
-                       unsigned long current, unsigned long total_arg)
-{
-  char buffer[60];
-  char units[] = "BKMGTPEZY?";
-  int unitidx = 0;
-  uint64_t total = total_arg;
-
-  /* Although we use an unsigned long for the values, 32 bit
-   * applications using GPGME will use an "int" and thus are limited
-   * in the total size which can be represented.  On Windows, where
-   * sizeof(int)==sizeof(long), this is even worse and will lead to an
-   * integer overflow for all files larger than 2 GiB.  Although, the
-   * allowed value range of TOTAL and CURRENT is nowhere specified, we
-   * better protect applications from the need to handle negative
-   * values.  The common usage pattern of the progress information is
-   * to display how many percent of the operation has been done and
-   * thus scaling CURRENT and TOTAL down before they get to large,
-   * should not have a noticeable effect except for rounding
-   * imprecision. */
-
-  if (!total && opt.input_size_hint)
-    total = opt.input_size_hint;
-
-  if (total)
-    {
-      if (current > total)
-        current = total;
-
-      while (total > 1024*1024)
-        {
-          total /= 1024;
-          current /= 1024;
-          unitidx++;
-        }
-    }
-  else
-    {
-      while (current > 1024*1024)
-        {
-          current /= 1024;
-          unitidx++;
-        }
-    }
-
-  if (unitidx > 9)
-    unitidx = 9;
-
-  snprintf (buffer, sizeof buffer, "%.20s ? %lu %lu %c%s",
-            what? what : "?", current, (unsigned long)total,
-            units[unitidx],
-            unitidx? "iB" : "");
-  write_status_text (STATUS_PROGRESS, buffer);
-}
-
-
 /****************
  * The filter is used to report progress to the user.
  */
@@ -140,11 +84,17 @@ progress_filter (void *opaque, int control,
 
   if (control == IOBUFCTRL_INIT)
     {
+      char buffer[50];
+
       pfx->last = 0;
       pfx->offset = 0;
       pfx->last_time = make_timestamp ();
 
-      write_status_progress (pfx->what, pfx->offset, pfx->total);
+      sprintf (buffer, "%.20s ? %lu %lu",
+               pfx->what? pfx->what : "?",
+               pfx->offset,
+	       pfx->total);
+      write_status_text (STATUS_PROGRESS, buffer);
     }
   else if (control == IOBUFCTRL_UNDERFLOW)
     {
@@ -164,7 +114,14 @@ progress_filter (void *opaque, int control,
       if ((len == -1 && pfx->offset != pfx->last)
 	  || timestamp - pfx->last_time > 0)
 	{
-          write_status_progress (pfx->what, pfx->offset, pfx->total);
+	  char buffer[50];
+	  
+	  sprintf (buffer, "%.20s ? %lu %lu",
+                   pfx->what? pfx->what : "?", 
+                   pfx->offset,
+		   pfx->total);
+	  write_status_text (STATUS_PROGRESS, buffer);
+
 	  pfx->last = pfx->offset;
 	  pfx->last_time = timestamp;
 	}
@@ -174,7 +131,7 @@ progress_filter (void *opaque, int control,
       release_progress_context (pfx);
     }
   else if (control == IOBUFCTRL_DESC)
-    mem2str (buf, "progress_filter", *ret_len);
+    *(char**)buf = "progress_filter";
   return rc;
 }
 
@@ -186,8 +143,8 @@ handle_progress (progress_filter_context_t *pfx, IOBUF inp, const char *name)
   if (!pfx)
     return;
 
-  log_assert (opt.enable_progress_filter);
-  log_assert (is_status_enabled ());
+  assert (opt.enable_progress_filter);
+  assert (is_status_enabled ());
 
   if ( !iobuf_is_pipe_filename (name) && *name )
     filesize = iobuf_get_filelength (inp, NULL);
