@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -23,6 +23,7 @@
 #include <string.h>
 #include <errno.h>
 #include <ctype.h>
+#include <assert.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -45,8 +46,8 @@
 		      (x) >= 'A' && (x) <= 'F' ? ((x)-'A'+10) : ((x)-'a'+10))
 
 
-/*
- * Write a record; die on error.
+/****************
+ * Wirte a record but die on error
  */
 static void
 write_record( TRUSTREC *rec )
@@ -55,34 +56,34 @@ write_record( TRUSTREC *rec )
     if( !rc )
 	return;
     log_error(_("trust record %lu, type %d: write failed: %s\n"),
-			    rec->recnum, rec->rectype, gpg_strerror (rc) );
+			    rec->recnum, rec->rectype, g10_errstr(rc) );
     tdbio_invalid();
 }
 
 
-/*
- * Dump the entire trustdb to FP or only the entries of one key.
+/****************
+ * Dump the entire trustdb or only the entries of one key.
  */
 void
-list_trustdb (estream_t fp, const char *username)
+list_trustdb( const char *username )
 {
   TRUSTREC rec;
-
+  
   (void)username;
-
-  init_trustdb (0);
+  
+  init_trustdb();
   /* For now we ignore the user ID. */
   if (1)
     {
       ulong recnum;
       int i;
-
-      es_fprintf (fp, "TrustDB: %s\n", tdbio_get_dbname ());
-      for (i = 9 + strlen (tdbio_get_dbname()); i > 0; i-- )
-        es_fputc ('-', fp);
-      es_putc ('\n', fp);
-      for (recnum=0; !tdbio_read_record (recnum, &rec, 0); recnum++)
-        tdbio_dump_record (&rec, fp);
+      
+      printf("TrustDB: %s\n", tdbio_get_dbname() );
+      for(i=9+strlen(tdbio_get_dbname()); i > 0; i-- )
+        putchar('-');
+      putchar('\n');
+      for(recnum=0; !tdbio_read_record( recnum, &rec, 0); recnum++ )
+        tdbio_dump_record( &rec, stdout );
     }
 }
 
@@ -96,25 +97,23 @@ list_trustdb (estream_t fp, const char *username)
 void
 export_ownertrust()
 {
-  TRUSTREC rec;
-  ulong recnum;
-  int i;
-  byte *p;
+    TRUSTREC rec;
+    ulong recnum;
+    int i;
+    byte *p;
 
-  init_trustdb (0);
-  es_printf (_("# List of assigned trustvalues, created %s\n"
-               "# (Use \"gpg --import-ownertrust\" to restore them)\n"),
-             asctimestamp( make_timestamp() ) );
-  for (recnum=0; !tdbio_read_record (recnum, &rec, 0); recnum++ )
-    {
-      if (rec.rectype == RECTYPE_TRUST)
-        {
-          if (!rec.r.trust.ownertrust)
-            continue;
-          p = rec.r.trust.fingerprint;
-          for (i=0; i < 20; i++, p++ )
-            es_printf("%02X", *p );
-          es_printf (":%u:\n", (unsigned int)rec.r.trust.ownertrust );
+    init_trustdb();
+    printf(_("# List of assigned trustvalues, created %s\n"
+	     "# (Use \"gpg --import-ownertrust\" to restore them)\n"),
+	   asctimestamp( make_timestamp() ) );
+    for(recnum=0; !tdbio_read_record( recnum, &rec, 0); recnum++ ) {
+	if( rec.rectype == RECTYPE_TRUST ) {
+	    if( !rec.r.trust.ownertrust )
+		continue;
+	    p = rec.r.trust.fingerprint;
+	    for(i=0; i < 20; i++, p++ )
+		printf("%02X", *p );
+	    printf(":%u:\n", (unsigned int)rec.r.trust.ownertrust );
 	}
     }
 }
@@ -123,7 +122,7 @@ export_ownertrust()
 void
 import_ownertrust( const char *fname )
 {
-    estream_t fp;
+    FILE *fp;
     int is_stdin=0;
     char line[256];
     char *p;
@@ -133,33 +132,33 @@ import_ownertrust( const char *fname )
     int any = 0;
     int rc;
 
-    init_trustdb (0);
+    init_trustdb();
     if( iobuf_is_pipe_filename (fname) ) {
-	fp = es_stdin;
+	fp = stdin;
 	fname = "[stdin]";
 	is_stdin = 1;
     }
-    else if( !(fp = es_fopen( fname, "r" )) ) {
-	log_error ( _("can't open '%s': %s\n"), fname, strerror(errno) );
+    else if( !(fp = fopen( fname, "r" )) ) {
+	log_error ( _("can't open `%s': %s\n"), fname, strerror(errno) );
 	return;
     }
 
-    if (is_secured_file (es_fileno (fp)))
+    if (is_secured_file (fileno (fp)))
       {
-        es_fclose (fp);
-        gpg_err_set_errno (EPERM);
-	log_error (_("can't open '%s': %s\n"), fname, strerror(errno) );
+        fclose (fp);
+        errno = EPERM;
+	log_error (_("can't open `%s': %s\n"), fname, strerror(errno) );
 	return;
       }
 
-    while (es_fgets (line, DIM(line)-1, fp)) {
+    while( fgets( line, DIM(line)-1, fp ) ) {
 	TRUSTREC rec;
 
 	if( !*line || *line == '#' )
 	    continue;
 	n = strlen(line);
 	if( line[n-1] != '\n' ) {
-	    log_error (_("error in '%s': %s\n"), fname, _("line too long") );
+	    log_error (_("error in `%s': %s\n"), fname, _("line too long") );
 	    /* ... or last line does not have a LF */
 	    break; /* can't continue */
 	}
@@ -167,17 +166,17 @@ import_ownertrust( const char *fname )
 	    if( !hexdigitp(p) )
 		break;
 	if( *p != ':' ) {
-	    log_error (_("error in '%s': %s\n"), fname, _("colon missing") );
+	    log_error (_("error in `%s': %s\n"), fname, _("colon missing") );
 	    continue;
 	}
 	fprlen = p - line;
 	if( fprlen != 32 && fprlen != 40 ) {
-	    log_error (_("error in '%s': %s\n"),
+	    log_error (_("error in `%s': %s\n"),
                        fname, _("invalid fingerprint") );
 	    continue;
 	}
 	if( sscanf(p, ":%u:", &otrust ) != 1 ) {
-	    log_error (_("error in '%s': %s\n"),
+	    log_error (_("error in `%s': %s\n"),
                        fname, _("ownertrust value missing"));
 	    continue;
 	}
@@ -188,27 +187,23 @@ import_ownertrust( const char *fname )
 	    fpr[fprlen++] = HEXTOBIN(p[0]) * 16 + HEXTOBIN(p[1]);
 	while (fprlen < 20)
 	    fpr[fprlen++] = 0;
-
+        
 	rc = tdbio_search_trust_byfpr (fpr, &rec);
 	if( !rc ) { /* found: update */
 	    if (rec.r.trust.ownertrust != otrust)
               {
-                if (!opt.quiet)
-                  {
-                    if( rec.r.trust.ownertrust )
-                      log_info("changing ownertrust from %u to %u\n",
-                               rec.r.trust.ownertrust, otrust );
-                    else
-                      log_info("setting ownertrust to %u\n", otrust );
-                  }
+                if( rec.r.trust.ownertrust )
+                  log_info("changing ownertrust from %u to %u\n",
+                           rec.r.trust.ownertrust, otrust );
+                else
+                  log_info("setting ownertrust to %u\n", otrust );
                 rec.r.trust.ownertrust = otrust;
                 write_record (&rec );
                 any = 1;
               }
 	}
-	else if (gpg_err_code (rc) == GPG_ERR_NOT_FOUND) { /* insert */
-            if (!opt.quiet)
-              log_info("inserting ownertrust of %u\n", otrust );
+	else if( rc == -1 ) { /* not found: insert */
+            log_info("inserting ownertrust of %u\n", otrust );
             memset (&rec, 0, sizeof rec);
             rec.recnum = tdbio_new_recnum ();
             rec.rectype = RECTYPE_TRUST;
@@ -218,20 +213,22 @@ import_ownertrust( const char *fname )
             any = 1;
 	}
 	else /* error */
-	    log_error (_("error finding trust record in '%s': %s\n"),
-                       fname, gpg_strerror (rc));
+	    log_error (_("error finding trust record in `%s': %s\n"),
+                       fname, g10_errstr(rc));
     }
-    if (es_ferror (fp))
-	log_error ( _("read error in '%s': %s\n"), fname, strerror(errno) );
-    if (!is_stdin)
-	es_fclose (fp);
-
+    if( ferror(fp) )
+	log_error ( _("read error in `%s': %s\n"), fname, strerror(errno) );
+    if( !is_stdin )
+	fclose(fp);
+    
     if (any)
       {
         revalidation_mark ();
         rc = tdbio_sync ();
         if (rc)
-          log_error (_("trustdb: sync failed: %s\n"), gpg_strerror (rc) );
+          log_error (_("trustdb: sync failed: %s\n"), g10_errstr(rc) );
       }
-
+    
 }
+
+
