@@ -1,4 +1,4 @@
-/* keybox-defs.h - internal Keybox definitions
+/* keybox-defs.h - interal Keybox defintions
  *	Copyright (C) 2001, 2004 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
@@ -21,12 +21,9 @@
 #define KEYBOX_DEFS_H 1
 
 #ifdef GPG_ERR_SOURCE_DEFAULT
-# if GPG_ERR_SOURCE_DEFAULT != GPG_ERR_SOURCE_KEYBOX
-#  error GPG_ERR_SOURCE_DEFAULT already defined
-# endif
-#else
-# define GPG_ERR_SOURCE_DEFAULT  GPG_ERR_SOURCE_KEYBOX
+#error GPG_ERR_SOURCE_DEFAULT already defined
 #endif
+#define GPG_ERR_SOURCE_DEFAULT  GPG_ERR_SOURCE_KEYBOX
 #include <gpg-error.h>
 #define map_assuan_err(a) \
         map_assuan_err_with_source (GPG_ERR_SOURCE_DEFAULT, (a))
@@ -37,19 +34,27 @@
    owns here.  This will not allow us build KBX in a standalone way
    but there is currently no need for it anyway.  Same goes for
    stringhelp.h which for example provides a replacement for stpcpy -
-   fixme: Better use the LIBOBJ mechnism. */
-#include "../common/types.h"
-#include "../common/stringhelp.h"
+   fixme: Better the LIBOBJ mechnism. */
+#include "../jnlib/types.h"
+#include "../jnlib/stringhelp.h"
 
 #include "keybox.h"
 
 
+enum {
+  BLOBTYPE_EMPTY = 0,
+  BLOBTYPE_HEADER = 1,
+  BLOBTYPE_PGP = 2,
+  BLOBTYPE_X509 = 3
+};
+
+
 typedef struct keyboxblob *KEYBOXBLOB;
 
 
 typedef struct keybox_name *KB_NAME;
 typedef struct keybox_name const *CONST_KB_NAME;
-struct keybox_name
+struct keybox_name 
 {
   /* Link to the next resources, so that we can walk all
      resources.  */
@@ -65,7 +70,7 @@ struct keybox_name
      entrues are set to NULL.  HANDLE_TABLE may be NULL. */
   KEYBOX_HANDLE *handle_table;
   size_t handle_table_size;
-
+  
   /* Not yet used.  */
   int is_locked;
 
@@ -77,14 +82,6 @@ struct keybox_name
 };
 
 
-struct keybox_found_s
-{
-  KEYBOXBLOB blob;
-  off_t offset;
-  size_t pk_no;
-  size_t uid_no;
-  unsigned int n_packets; /*used for delete and update*/
-};
 
 struct keybox_handle {
   CONST_KB_NAME kb;
@@ -92,10 +89,14 @@ struct keybox_handle {
   FILE *fp;
   int eof;
   int error;
-  int ephemeral;
-  int for_openpgp;        /* Used by gpg.  */
-  struct keybox_found_s found;
-  struct keybox_found_s saved_found;
+  int ephemeral;  
+  struct {
+    KEYBOXBLOB blob;
+    off_t offset;
+    size_t pk_no;
+    size_t uid_no;
+    unsigned int n_packets; /*used for delete and update*/
+  } found;
   struct {
     char *name;
     char *pattern;
@@ -107,7 +108,6 @@ struct keybox_handle {
 struct _keybox_openpgp_key_info
 {
   struct _keybox_openpgp_key_info *next;
-  int algo;
   unsigned char keyid[8];
   int fprlen;  /* Either 16 or 20 */
   unsigned char fpr[20];
@@ -152,12 +152,9 @@ void _keybox_close_file (KEYBOX_HANDLE hd);
 
 
 /*-- keybox-blob.c --*/
-gpg_error_t _keybox_create_openpgp_blob (KEYBOXBLOB *r_blob,
-                                         keybox_openpgp_info_t info,
-                                         const unsigned char *image,
-                                         size_t imagelen,
-                                         u32 *sigstatus,
-                                         int as_ephemeral);
+#ifdef KEYBOX_WITH_OPENPGP
+  /* fixme */
+#endif /*KEYBOX_WITH_OPENPGP*/
 #ifdef KEYBOX_WITH_X509
 int _keybox_create_x509_blob (KEYBOXBLOB *r_blob, ksba_cert_t cert,
                               unsigned char *sha1_digest, int as_ephemeral);
@@ -169,7 +166,7 @@ int  _keybox_new_blob (KEYBOXBLOB *r_blob,
 void _keybox_release_blob (KEYBOXBLOB blob);
 const unsigned char *_keybox_get_blob_image (KEYBOXBLOB blob, size_t *n);
 off_t _keybox_get_blob_fileoffset (KEYBOXBLOB blob);
-void _keybox_update_header_blob (KEYBOXBLOB blob, int for_openpgp);
+void _keybox_update_header_blob (KEYBOXBLOB blob);
 
 /*-- keybox-openpgp.c --*/
 gpg_error_t _keybox_parse_openpgp (const unsigned char *image, size_t imagelen,
@@ -182,6 +179,7 @@ void _keybox_destroy_openpgp_info (keybox_openpgp_info_t info);
 int _keybox_read_blob (KEYBOXBLOB *r_blob, FILE *fp);
 int _keybox_read_blob2 (KEYBOXBLOB *r_blob, FILE *fp, int *skipped_deleted);
 int _keybox_write_blob (KEYBOXBLOB blob, FILE *fp);
+int _keybox_write_header_blob (FILE *fp);
 
 /*-- keybox-search.c --*/
 gpg_err_code_t _keybox_get_flag_location (const unsigned char *buffer,
@@ -189,20 +187,6 @@ gpg_err_code_t _keybox_get_flag_location (const unsigned char *buffer,
                                           int what,
                                           size_t *flag_off, size_t *flag_size);
 
-static inline int
-blob_get_type (KEYBOXBLOB blob)
-{
-  const unsigned char *buffer;
-  size_t length;
-
-  buffer = _keybox_get_blob_image (blob, &length);
-  if (length < 32)
-    return -1; /* blob too short */
-
-  return buffer[4];
-}
-
-
 /*-- keybox-dump.c --*/
 int _keybox_dump_blob (KEYBOXBLOB blob, FILE *fp);
 int _keybox_dump_file (const char *filename, int stats_only, FILE *outfp);
@@ -231,24 +215,24 @@ void  _keybox_free (void *p);
 #define STR2(v) STR(v)
 
 /*
-  a couple of handy macros
+  a couple of handy macros 
 */
 
 #define return_if_fail(expr) do {                        \
     if (!(expr)) {                                       \
-        fprintf (stderr, "%s:%d: assertion '%s' failed\n", \
+        fprintf (stderr, "%s:%d: assertion `%s' failed\n", \
                  __FILE__, __LINE__, #expr );            \
         return;	                                         \
     } } while (0)
 #define return_null_if_fail(expr) do {                   \
     if (!(expr)) {                                       \
-        fprintf (stderr, "%s:%d: assertion '%s' failed\n", \
+        fprintf (stderr, "%s:%d: assertion `%s' failed\n", \
                  __FILE__, __LINE__, #expr );            \
         return NULL;	                                 \
     } } while (0)
 #define return_val_if_fail(expr,val) do {                \
     if (!(expr)) {                                       \
-        fprintf (stderr, "%s:%d: assertion '%s' failed\n", \
+        fprintf (stderr, "%s:%d: assertion `%s' failed\n", \
                  __FILE__, __LINE__, #expr );            \
         return (val);	                                 \
     } } while (0)
@@ -273,3 +257,5 @@ void  _keybox_free (void *p);
 
 
 #endif /*KEYBOX_DEFS_H*/
+
+
