@@ -1,5 +1,5 @@
 /* keybox.h - Keybox operations
- * Copyright (C) 2001, 2003, 2012 Free Software Foundation, Inc.
+ *	Copyright (C) 2001, 2003 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -20,15 +20,23 @@
 #ifndef KEYBOX_H
 #define KEYBOX_H 1
 #ifdef __cplusplus
-extern "C" {
+extern "C" { 
 #if 0
  }
 #endif
 #endif
 
-#include "../common/iobuf.h"
 #include "keybox-search-desc.h"
 
+#define KEYBOX_WITH_OPENPGP 1 
+#define KEYBOX_WITH_X509 1
+
+
+#ifdef KEYBOX_WITH_OPENPGP
+#  undef KEYBOX_WITH_OPENPGP
+/*#include <lib-to-handle-gpg-data-structs.h>*/
+#endif
+
 #ifdef KEYBOX_WITH_X509
 # include <ksba.h>
 #endif
@@ -45,65 +53,37 @@ typedef enum
     KEYBOX_FLAG_UID,        /* The user ID flags; requires an uid index. */
     KEYBOX_FLAG_UID_VALIDITY,/* The validity of a specific uid, requires
                                an uid index. */
-    KEYBOX_FLAG_CREATED_AT, /* The date the block was created. */
-    KEYBOX_FLAG_SIG_INFO,   /* The signature info block.  */
+    KEYBOX_FLAG_CREATED_AT  /* The date the block was created. */
   } keybox_flag_t;
 
 /* Flag values used with KEYBOX_FLAG_BLOB.  */
 #define KEYBOX_FLAG_BLOB_SECRET     1
 #define KEYBOX_FLAG_BLOB_EPHEMERAL  2
 
-/* The keybox blob types.  */
-typedef enum
-  {
-    KEYBOX_BLOBTYPE_EMPTY  = 0,
-    KEYBOX_BLOBTYPE_HEADER = 1,
-    KEYBOX_BLOBTYPE_PGP    = 2,
-    KEYBOX_BLOBTYPE_X509   = 3
-  } keybox_blobtype_t;
 
 
 /*-- keybox-init.c --*/
 void *keybox_register_file (const char *fname, int secret);
 int keybox_is_writable (void *token);
 
-KEYBOX_HANDLE keybox_new_openpgp (void *token, int secret);
-KEYBOX_HANDLE keybox_new_x509 (void *token, int secret);
+KEYBOX_HANDLE keybox_new (void *token, int secret);
 void keybox_release (KEYBOX_HANDLE hd);
-void keybox_push_found_state (KEYBOX_HANDLE hd);
-void keybox_pop_found_state (KEYBOX_HANDLE hd);
 const char *keybox_get_resource_name (KEYBOX_HANDLE hd);
 int keybox_set_ephemeral (KEYBOX_HANDLE hd, int yes);
 
-int keybox_lock (KEYBOX_HANDLE hd, int yes);
-
-/*-- keybox-file.c --*/
-/* Fixme: This function does not belong here: Provide a better
-   interface to create a new keybox file.  */
-int _keybox_write_header_blob (FILE *fp, int openpgp_flag);
 
 /*-- keybox-search.c --*/
-gpg_error_t keybox_get_keyblock (KEYBOX_HANDLE hd, iobuf_t *r_iobuf,
-                                 int *r_uid_no, int *r_pk_no, u32 **sigstatus);
-#ifdef KEYBOX_WITH_X509
+#ifdef KEYBOX_WITH_X509 
 int keybox_get_cert (KEYBOX_HANDLE hd, ksba_cert_t *ret_cert);
 #endif /*KEYBOX_WITH_X509*/
 int keybox_get_flags (KEYBOX_HANDLE hd, int what, int idx, unsigned int *value);
 
 int keybox_search_reset (KEYBOX_HANDLE hd);
-int keybox_search (KEYBOX_HANDLE hd, KEYBOX_SEARCH_DESC *desc, size_t ndesc,
-                   keybox_blobtype_t want_blobtype,
-                   size_t *r_descindex, unsigned long *r_skipped);
+int keybox_search (KEYBOX_HANDLE hd, KEYBOX_SEARCH_DESC *desc, size_t ndesc);
 
 
 /*-- keybox-update.c --*/
-gpg_error_t keybox_insert_keyblock (KEYBOX_HANDLE hd,
-                                    const void *image, size_t imagelen,
-                                    u32 *sigstatus);
-gpg_error_t keybox_update_keyblock (KEYBOX_HANDLE hd,
-                                    const void *image, size_t imagelen);
-
-#ifdef KEYBOX_WITH_X509
+#ifdef KEYBOX_WITH_X509 
 int keybox_insert_cert (KEYBOX_HANDLE hd, ksba_cert_t cert,
                         unsigned char *sha1_digest);
 int keybox_update_cert (KEYBOX_HANDLE hd, ksba_cert_t cert,
@@ -118,7 +98,11 @@ int keybox_compress (KEYBOX_HANDLE hd);
 /*--  --*/
 
 #if 0
+int keybox_lock (KEYBOX_HANDLE hd, int yes);
+int keybox_get_keyblock (KEYBOX_HANDLE hd, KBNODE *ret_kb);
 int keybox_locate_writable (KEYBOX_HANDLE hd);
+int keybox_search_reset (KEYBOX_HANDLE hd);
+int keybox_search (KEYBOX_HANDLE hd, KEYDB_SEARCH_DESC *desc, size_t ndesc);
 int keybox_rebuild_cache (void *);
 #endif
 
