@@ -273,7 +273,7 @@ read_32 (FILE *fp)
       fprintf (stderr, PGM ": premature EOF while parsing request\n");
       exit (1);
     }
-  return ((unsigned long)c1 << 24) | (c2 << 16) | (c3 << 8) | c4;
+  return (c1 << 24) | (c2 << 16) | (c3 << 8) | c4;
 }
 
 
@@ -714,7 +714,7 @@ handle_transmit (unsigned char *argbuf, size_t arglen)
   long err;
   struct pcsc_io_request_s send_pci;
   pcsc_dword_t recv_len;
-  unsigned char buffer[4096];
+  unsigned char buffer[1024];
 
   /* The apdu should at least be one byte. */
   if (!arglen)
@@ -760,8 +760,7 @@ handle_control (unsigned char *argbuf, size_t arglen)
   if (arglen < 4)
     bad_request ("CONTROL");
 
-  ioctl_code = (((pcsc_dword_t)argbuf[0] << 24)
-                | (argbuf[1] << 16) | (argbuf[2] << 8) | argbuf[3]);
+  ioctl_code = (argbuf[0] << 24) | (argbuf[1] << 16) | (argbuf[2] << 8) | argbuf[3];
   argbuf += 4;
   arglen -= 4;
 
