@@ -1,6 +1,6 @@
 /* call-agent.c - Divert GPGSM operations to the agent
  * Copyright (C) 2001, 2002, 2003, 2005, 2007,
- *               2008, 2009, 2010 Free Software Foundation, Inc.
+ *               2008, 2009 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -23,7 +23,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <unistd.h>
+#include <unistd.h> 
 #include <time.h>
 #include <assert.h>
 #ifdef HAVE_LOCALE_H
@@ -33,12 +33,10 @@
 #include "gpgsm.h"
 #include <gcrypt.h>
 #include <assuan.h>
-#include "../common/i18n.h"
-#include "../common/asshelp.h"
+#include "i18n.h"
+#include "asshelp.h"
 #include "keydb.h" /* fixme: Move this to import.c */
-#include "../common/membuf.h"
-#include "../common/shareddefs.h"
-#include "passphrase.h"
+#include "membuf.h"
 
 
 static assuan_context_t agent_ctx = NULL;
@@ -68,63 +66,8 @@ struct learn_parm_s
   membuf_t *data;
 };
 
-struct import_key_parm_s
-{
-  ctrl_t ctrl;
-  assuan_context_t ctx;
-  const void *key;
-  size_t keylen;
-};
-
-struct default_inq_parm_s
-{
-  ctrl_t ctrl;
-  assuan_context_t ctx;
-};
 
 
-/* Print a warning if the server's version number is less than our
-   version number.  Returns an error code on a connection problem.  */
-static gpg_error_t
-warn_version_mismatch (ctrl_t ctrl, assuan_context_t ctx,
-                       const char *servername, int mode)
-{
-  gpg_error_t err;
-  char *serverversion;
-  const char *myversion = strusage (13);
-
-  err = get_assuan_server_version (ctx, mode, &serverversion);
-  if (err)
-    log_error (_("error getting version from '%s': %s\n"),
-               servername, gpg_strerror (err));
-  else if (compare_version_strings (serverversion, myversion) < 0)
-    {
-      char *warn;
-
-      warn = xtryasprintf (_("server '%s' is older than us (%s < %s)"),
-                           servername, serverversion, myversion);
-      if (!warn)
-        err = gpg_error_from_syserror ();
-      else
-        {
-          log_info (_("WARNING: %s\n"), warn);
-          if (!opt.quiet)
-            {
-              log_info (_("Note: Outdated servers may lack important"
-                          " security fixes.\n"));
-              log_info (_("Note: Use the command \"%s\" to restart them.\n"),
-                        "gpgconf --kill all");
-            }
-          gpgsm_status2 (ctrl, STATUS_WARNING, "server_version_mismatch 0",
-                         warn, NULL);
-          xfree (warn);
-        }
-    }
-  xfree (serverversion);
-  return err;
-}
-
-
 /* Try to connect to the agent via socket or fork it off and work by
    pipes.  Handle the server's initial greeting */
 static int
@@ -140,63 +83,20 @@ start_agent (ctrl_t ctrl)
     {
       rc = start_new_gpg_agent (&agent_ctx,
                                 GPG_ERR_SOURCE_DEFAULT,
+                                opt.homedir,
                                 opt.agent_program,
                                 opt.lc_ctype, opt.lc_messages,
                                 opt.session_env,
-                                opt.autostart, opt.verbose, DBG_IPC,
+                                opt.verbose, DBG_ASSUAN,
                                 gpgsm_status2, ctrl);
-
-      if (!opt.autostart && gpg_err_code (rc) == GPG_ERR_NO_AGENT)
-        {
-          static int shown;
-
-          if (!shown)
-            {
-              shown = 1;
-              log_info (_("no gpg-agent running in this session\n"));
-            }
-        }
-      else if (!rc && !(rc = warn_version_mismatch (ctrl, agent_ctx,
-                                                    GPG_AGENT_NAME, 0)))
+      
+      if (!rc)
         {
           /* Tell the agent that we support Pinentry notifications.  No
              error checking so that it will work also with older
              agents.  */
           assuan_transact (agent_ctx, "OPTION allow-pinentry-notify",
                            NULL, NULL, NULL, NULL, NULL, NULL);
-
-          /* Pass on the pinentry mode.  */
-          if (opt.pinentry_mode)
-            {
-              char *tmp = xasprintf ("OPTION pinentry-mode=%s",
-                                     str_pinentry_mode (opt.pinentry_mode));
-              rc = assuan_transact (agent_ctx, tmp,
-                               NULL, NULL, NULL, NULL, NULL, NULL);
-              xfree (tmp);
-              if (rc)
-                log_error ("setting pinentry mode '%s' failed: %s\n",
-                           str_pinentry_mode (opt.pinentry_mode),
-                           gpg_strerror (rc));
-            }
-
-          /* In DE_VS mode under Windows we require that the JENT RNG
-           * is active.  */
-#ifdef HAVE_W32_SYSTEM
-          if (!rc && opt.compliance == CO_DE_VS)
-            {
-              if (assuan_transact (agent_ctx, "GETINFO jent_active",
-                                   NULL, NULL, NULL, NULL, NULL, NULL))
-                {
-                  rc = gpg_error (GPG_ERR_FORBIDDEN);
-                  log_error (_("%s is not compliant with %s mode\n"),
-                             GPG_AGENT_NAME,
-                             gnupg_compliance_option_string (opt.compliance));
-                  gpgsm_status_with_error (ctrl, STATUS_ERROR,
-                                           "random-compliance", rc);
-                }
-            }
-#endif /*HAVE_W32_SYSTEM*/
-
         }
     }
 
@@ -209,35 +109,39 @@ start_agent (ctrl_t ctrl)
   return rc;
 }
 
+
+
+static gpg_error_t
+membuf_data_cb (void *opaque, const void *buffer, size_t length)
+{
+  membuf_t *data = opaque;
+
+  if (buffer)
+    put_membuf (data, buffer, length);
+  return 0;
+}
+  
+
 /* This is the default inquiry callback.  It mainly handles the
    Pinentry notifications.  */
 static gpg_error_t
 default_inq_cb (void *opaque, const char *line)
 {
-  gpg_error_t err = 0;
-  struct default_inq_parm_s *parm = opaque;
-  ctrl_t ctrl = parm->ctrl;
+  gpg_error_t err;
+  ctrl_t ctrl = opaque;
 
-  if (has_leading_keyword (line, "PINENTRY_LAUNCHED"))
+  if (!strncmp (line, "PINENTRY_LAUNCHED", 17) && (line[17]==' '||!line[17]))
     {
       err = gpgsm_proxy_pinentry_notify (ctrl, line);
       if (err)
-        log_error (_("failed to proxy %s inquiry to client\n"),
+        log_error (_("failed to proxy %s inquiry to client\n"), 
                    "PINENTRY_LAUNCHED");
       /* We do not pass errors to avoid breaking other code.  */
     }
-  else if ((has_leading_keyword (line, "PASSPHRASE")
-            || has_leading_keyword (line, "NEW_PASSPHRASE"))
-           && opt.pinentry_mode == PINENTRY_MODE_LOOPBACK
-           && have_static_passphrase ())
-    {
-      const char *s = get_static_passphrase ();
-      err = assuan_send_data (parm->ctx, s, strlen (s));
-    }
   else
-    log_error ("ignoring gpg-agent inquiry '%s'\n", line);
+    log_error ("ignoring gpg-agent inquiry `%s'\n", line);
 
-  return err;
+  return 0;
 }
 
 
@@ -254,14 +158,11 @@ gpgsm_agent_pksign (ctrl_t ctrl, const char *keygrip, const char *desc,
   char *p, line[ASSUAN_LINELENGTH];
   membuf_t data;
   size_t len;
-  struct default_inq_parm_s inq_parm;
 
   *r_buf = NULL;
   rc = start_agent (ctrl);
   if (rc)
     return rc;
-  inq_parm.ctrl = ctrl;
-  inq_parm.ctx = agent_ctx;
 
   if (digestlen*2 + 50 > DIM(line))
     return gpg_error (GPG_ERR_GENERAL);
@@ -270,14 +171,16 @@ gpgsm_agent_pksign (ctrl_t ctrl, const char *keygrip, const char *desc,
   if (rc)
     return rc;
 
-  snprintf (line, DIM(line), "SIGKEY %s", keygrip);
+  snprintf (line, DIM(line)-1, "SIGKEY %s", keygrip);
+  line[DIM(line)-1] = 0;
   rc = assuan_transact (agent_ctx, line, NULL, NULL, NULL, NULL, NULL, NULL);
   if (rc)
     return rc;
 
   if (desc)
     {
-      snprintf (line, DIM(line), "SETKEYDESC %s", desc);
+      snprintf (line, DIM(line)-1, "SETKEYDESC %s", desc);
+      line[DIM(line)-1] = 0;
       rc = assuan_transact (agent_ctx, line,
                             NULL, NULL, NULL, NULL, NULL, NULL);
       if (rc)
@@ -294,7 +197,7 @@ gpgsm_agent_pksign (ctrl_t ctrl, const char *keygrip, const char *desc,
 
   init_membuf (&data, 1024);
   rc = assuan_transact (agent_ctx, "PKSIGN",
-                        put_membuf_cb, &data, default_inq_cb, &inq_parm,
+                        membuf_data_cb, &data, default_inq_cb, ctrl,
                         NULL, NULL);
   if (rc)
     {
@@ -327,7 +230,6 @@ gpgsm_scd_pksign (ctrl_t ctrl, const char *keyid, const char *desc,
   const char *hashopt;
   unsigned char *sigbuf;
   size_t sigbuflen;
-  struct default_inq_parm_s inq_parm;
 
   (void)desc;
 
@@ -339,15 +241,13 @@ gpgsm_scd_pksign (ctrl_t ctrl, const char *keyid, const char *desc,
     case GCRY_MD_RMD160:hashopt = "--hash=rmd160"; break;
     case GCRY_MD_MD5:   hashopt = "--hash=md5"; break;
     case GCRY_MD_SHA256:hashopt = "--hash=sha256"; break;
-    default:
+    default: 
       return gpg_error (GPG_ERR_DIGEST_ALGO);
     }
 
   rc = start_agent (ctrl);
   if (rc)
     return rc;
-  inq_parm.ctrl = ctrl;
-  inq_parm.ctx = agent_ctx;
 
   if (digestlen*2 + 50 > DIM(line))
     return gpg_error (GPG_ERR_GENERAL);
@@ -361,9 +261,10 @@ gpgsm_scd_pksign (ctrl_t ctrl, const char *keyid, const char *desc,
 
   init_membuf (&data, 1024);
 
-  snprintf (line, DIM(line), "SCD PKSIGN %s %s", hashopt, keyid);
+  snprintf (line, DIM(line)-1, "SCD PKSIGN %s %s", hashopt, keyid);
+  line[DIM(line)-1] = 0;
   rc = assuan_transact (agent_ctx, line,
-                        put_membuf_cb, &data, default_inq_cb, &inq_parm,
+                        membuf_data_cb, &data, default_inq_cb, ctrl,
                         NULL, NULL);
   if (rc)
     {
@@ -399,26 +300,23 @@ gpgsm_scd_pksign (ctrl_t ctrl, const char *keyid, const char *desc,
 
 
 /* Handle a CIPHERTEXT inquiry.  Note, we only send the data,
-   assuan_transact takes care of flushing and writing the end */
+   assuan_transact talkes care of flushing and writing the end */
 static gpg_error_t
 inq_ciphertext_cb (void *opaque, const char *line)
 {
-  struct cipher_parm_s *parm = opaque;
+  struct cipher_parm_s *parm = opaque; 
   int rc;
 
-  if (has_leading_keyword (line, "CIPHERTEXT"))
+  if (!strncmp (line, "CIPHERTEXT", 10) && (line[10]==' '||!line[10]))
     {
       assuan_begin_confidential (parm->ctx);
       rc = assuan_send_data (parm->ctx, parm->ciphertext, parm->ciphertextlen);
       assuan_end_confidential (parm->ctx);
     }
   else
-    {
-      struct default_inq_parm_s inq_parm = { parm->ctrl, parm->ctx };
-      rc = default_inq_cb (&inq_parm, line);
-    }
+    rc = default_inq_cb (parm->ctrl, line);
 
-  return rc;
+  return rc; 
 }
 
 
@@ -426,7 +324,7 @@ inq_ciphertext_cb (void *opaque, const char *line)
    the hex string KEYGRIP. */
 int
 gpgsm_agent_pkdecrypt (ctrl_t ctrl, const char *keygrip, const char *desc,
-                       ksba_const_sexp_t ciphertext,
+                       ksba_const_sexp_t ciphertext, 
                        char **r_buf, size_t *r_buflen )
 {
   int rc;
@@ -436,7 +334,7 @@ gpgsm_agent_pkdecrypt (ctrl_t ctrl, const char *keygrip, const char *desc,
   size_t n, len;
   char *p, *buf, *endp;
   size_t ciphertextlen;
-
+  
   if (!keygrip || strlen(keygrip) != 40 || !ciphertext || !r_buf || !r_buflen)
     return gpg_error (GPG_ERR_INV_VALUE);
   *r_buf = NULL;
@@ -454,14 +352,16 @@ gpgsm_agent_pkdecrypt (ctrl_t ctrl, const char *keygrip, const char *desc,
     return rc;
 
   assert ( DIM(line) >= 50 );
-  snprintf (line, DIM(line), "SETKEY %s", keygrip);
+  snprintf (line, DIM(line)-1, "SETKEY %s", keygrip);
+  line[DIM(line)-1] = 0;
   rc = assuan_transact (agent_ctx, line, NULL, NULL, NULL, NULL, NULL, NULL);
   if (rc)
     return rc;
 
   if (desc)
     {
-      snprintf (line, DIM(line), "SETKEYDESC %s", desc);
+      snprintf (line, DIM(line)-1, "SETKEYDESC %s", desc);
+      line[DIM(line)-1] = 0;
       rc = assuan_transact (agent_ctx, line,
                             NULL, NULL, NULL, NULL, NULL, NULL);
       if (rc)
@@ -474,7 +374,7 @@ gpgsm_agent_pkdecrypt (ctrl_t ctrl, const char *keygrip, const char *desc,
   cipher_parm.ciphertext = ciphertext;
   cipher_parm.ciphertextlen = ciphertextlen;
   rc = assuan_transact (agent_ctx, "PKDECRYPT",
-                        put_membuf_cb, &data,
+                        membuf_data_cb, &data,
                         inq_ciphertext_cb, &cipher_parm, NULL, NULL);
   if (rc)
     {
@@ -509,7 +409,7 @@ gpgsm_agent_pkdecrypt (ctrl_t ctrl, const char *keygrip, const char *desc,
   endp++;
   if (endp-p+n > len)
     return gpg_error (GPG_ERR_INV_SEXP); /* Oops: Inconsistent S-Exp. */
-
+  
   memmove (buf, endp, n);
 
   *r_buflen = n;
@@ -526,20 +426,17 @@ gpgsm_agent_pkdecrypt (ctrl_t ctrl, const char *keygrip, const char *desc,
 static gpg_error_t
 inq_genkey_parms (void *opaque, const char *line)
 {
-  struct genkey_parm_s *parm = opaque;
+  struct genkey_parm_s *parm = opaque; 
   int rc;
 
-  if (has_leading_keyword (line, "KEYPARAM"))
+  if (!strncmp (line, "KEYPARAM", 8) && (line[8]==' '||!line[8]))
     {
       rc = assuan_send_data (parm->ctx, parm->sexp, parm->sexplen);
     }
   else
-    {
-      struct default_inq_parm_s inq_parm = { parm->ctrl, parm->ctx };
-      rc = default_inq_cb (&inq_parm, line);
-    }
+    rc = default_inq_cb (parm->ctrl, line);
 
-  return rc;
+  return rc; 
 }
 
 
@@ -572,7 +469,7 @@ gpgsm_agent_genkey (ctrl_t ctrl,
   if (!gk_parm.sexplen)
     return gpg_error (GPG_ERR_INV_VALUE);
   rc = assuan_transact (agent_ctx, "GENKEY",
-                        put_membuf_cb, &data,
+                        membuf_data_cb, &data, 
                         inq_genkey_parms, &gk_parm, NULL, NULL);
   if (rc)
     {
@@ -605,26 +502,24 @@ gpgsm_agent_readkey (ctrl_t ctrl, int fromcard, const char *hexkeygrip,
   size_t len;
   unsigned char *buf;
   char line[ASSUAN_LINELENGTH];
-  struct default_inq_parm_s inq_parm;
 
   *r_pubkey = NULL;
   rc = start_agent (ctrl);
   if (rc)
     return rc;
-  inq_parm.ctrl = ctrl;
-  inq_parm.ctx = agent_ctx;
 
   rc = assuan_transact (agent_ctx, "RESET",NULL, NULL, NULL, NULL, NULL, NULL);
   if (rc)
     return rc;
 
-  snprintf (line, DIM(line), "%sREADKEY %s",
+  snprintf (line, DIM(line)-1, "%sREADKEY %s",
             fromcard? "SCD ":"", hexkeygrip);
+  line[DIM(line)-1] = 0;
 
   init_membuf (&data, 1024);
   rc = assuan_transact (agent_ctx, line,
-                        put_membuf_cb, &data,
-                        default_inq_cb, &inq_parm, NULL, NULL);
+                        membuf_data_cb, &data, 
+                        default_inq_cb, ctrl, NULL, NULL);
   if (rc)
     {
       xfree (get_membuf (&data, &len));
@@ -665,7 +560,7 @@ store_serialno (const char *line)
 }
 
 
-/* Callback for the gpgsm_agent_serialno function.  */
+/* Callback for the gpgsm_agent_serialno fucntion.  */
 static gpg_error_t
 scd_serialno_status_cb (void *opaque, const char *line)
 {
@@ -694,18 +589,15 @@ gpgsm_agent_scd_serialno (ctrl_t ctrl, char **r_serialno)
 {
   int rc;
   char *serialno = NULL;
-  struct default_inq_parm_s inq_parm;
-
+ 
   *r_serialno = NULL;
   rc = start_agent (ctrl);
   if (rc)
     return rc;
-  inq_parm.ctrl = ctrl;
-  inq_parm.ctx = agent_ctx;
 
   rc = assuan_transact (agent_ctx, "SCD SERIALNO",
                         NULL, NULL,
-                        default_inq_cb, &inq_parm,
+                        default_inq_cb, ctrl,
                         scd_serialno_status_cb, &serialno);
   if (!rc && !serialno)
     rc = gpg_error (GPG_ERR_INTERNAL);
@@ -720,7 +612,7 @@ gpgsm_agent_scd_serialno (ctrl_t ctrl, char **r_serialno)
 
 
 
-/* Callback for the gpgsm_agent_serialno function.  */
+/* Callback for the gpgsm_agent_serialno fucntion.  */
 static gpg_error_t
 scd_keypairinfo_status_cb (void *opaque, const char *line)
 {
@@ -766,18 +658,15 @@ gpgsm_agent_scd_keypairinfo (ctrl_t ctrl, strlist_t *r_list)
 {
   int rc;
   strlist_t list = NULL;
-  struct default_inq_parm_s inq_parm;
-
+ 
   *r_list = NULL;
   rc = start_agent (ctrl);
   if (rc)
     return rc;
-  inq_parm.ctrl = ctrl;
-  inq_parm.ctx = agent_ctx;
 
   rc = assuan_transact (agent_ctx, "SCD LEARN --force",
                         NULL, NULL,
-                        default_inq_cb, &inq_parm,
+                        default_inq_cb, ctrl,
                         scd_keypairinfo_status_cb, &list);
   if (!rc && !list)
     rc = gpg_error (GPG_ERR_NO_DATA);
@@ -796,14 +685,14 @@ static gpg_error_t
 istrusted_status_cb (void *opaque, const char *line)
 {
   struct rootca_flags_s *flags = opaque;
-  const char *s;
 
-  if ((s = has_leading_keyword (line, "TRUSTLISTFLAG")))
+  if (!strncmp (line, "TRUSTLISTFLAG", 13) && (line[13]==' ' || !line[13]))
     {
-      line = s;
-      if (has_leading_keyword (line, "relax"))
+      for (line += 13; *line == ' '; line++)
+        ;
+      if (!strncmp (line, "relax", 5) && (line[5] == ' ' || !line[5]))
         flags->relax = 1;
-      else if (has_leading_keyword (line, "cm"))
+      else if (!strncmp (line, "cm", 2) && (line[2] == ' ' || !line[2]))
         flags->chain_model = 1;
     }
   return 0;
@@ -833,7 +722,8 @@ gpgsm_agent_istrusted (ctrl_t ctrl, ksba_cert_t cert, const char *hexfpr,
 
   if (hexfpr)
     {
-      snprintf (line, DIM(line), "ISTRUSTED %s", hexfpr);
+      snprintf (line, DIM(line)-1, "ISTRUSTED %s", hexfpr);
+      line[DIM(line)-1] = 0;
     }
   else
     {
@@ -845,8 +735,9 @@ gpgsm_agent_istrusted (ctrl_t ctrl, ksba_cert_t cert, const char *hexfpr,
           log_error ("error getting the fingerprint\n");
           return gpg_error (GPG_ERR_GENERAL);
         }
-
-      snprintf (line, DIM(line), "ISTRUSTED %s", fpr);
+      
+      snprintf (line, DIM(line)-1, "ISTRUSTED %s", fpr);
+      line[DIM(line)-1] = 0;
       xfree (fpr);
     }
 
@@ -864,13 +755,10 @@ gpgsm_agent_marktrusted (ctrl_t ctrl, ksba_cert_t cert)
   int rc;
   char *fpr, *dn, *dnfmt;
   char line[ASSUAN_LINELENGTH];
-  struct default_inq_parm_s inq_parm;
 
   rc = start_agent (ctrl);
   if (rc)
     return rc;
-  inq_parm.ctrl = ctrl;
-  inq_parm.ctx = agent_ctx;
 
   fpr = gpgsm_get_fingerprint_hexstring (cert, GCRY_MD_SHA1);
   if (!fpr)
@@ -889,12 +777,13 @@ gpgsm_agent_marktrusted (ctrl_t ctrl, ksba_cert_t cert)
   xfree (dn);
   if (!dnfmt)
     return gpg_error_from_syserror ();
-  snprintf (line, DIM(line), "MARKTRUSTED %s S %s", fpr, dnfmt);
+  snprintf (line, DIM(line)-1, "MARKTRUSTED %s S %s", fpr, dnfmt);
+  line[DIM(line)-1] = 0;
   ksba_free (dnfmt);
   xfree (fpr);
 
   rc = assuan_transact (agent_ctx, line, NULL, NULL,
-                        default_inq_cb, &inq_parm, NULL, NULL);
+                        default_inq_cb, ctrl, NULL, NULL);
   return rc;
 }
 
@@ -915,7 +804,8 @@ gpgsm_agent_havekey (ctrl_t ctrl, const char *hexkeygrip)
   if (!hexkeygrip || strlen (hexkeygrip) != 40)
     return gpg_error (GPG_ERR_INV_VALUE);
 
-  snprintf (line, DIM(line), "HAVEKEY %s", hexkeygrip);
+  snprintf (line, DIM(line)-1, "HAVEKEY %s", hexkeygrip);
+  line[DIM(line)-1] = 0;
 
   rc = assuan_transact (agent_ctx, line, NULL, NULL, NULL, NULL, NULL, NULL);
   return rc;
@@ -926,14 +816,14 @@ static gpg_error_t
 learn_status_cb (void *opaque, const char *line)
 {
   struct learn_parm_s *parm = opaque;
-  const char *s;
 
   /* Pass progress data to the caller.  */
-  if ((s = has_leading_keyword (line, "PROGRESS")))
+  if (!strncmp (line, "PROGRESS", 8) && (line[8]==' ' || !line[8]))
     {
-      line = s;
       if (parm->ctrl)
         {
+          for (line += 8; *line == ' '; line++)
+            ;
           if (gpgsm_status (parm->ctrl, STATUS_PROGRESS, line))
             return gpg_error (GPG_ERR_ASS_CANCELED);
         }
@@ -996,7 +886,7 @@ learn_cb (void *opaque, const void *buffer, size_t length)
     {
       int existed;
 
-      if (!keydb_store_cert (parm->ctrl, cert, 0, &existed))
+      if (!keydb_store_cert (cert, 0, &existed))
         {
           if (opt.verbose > 1 && existed)
             log_info ("certificate already in DB\n");
@@ -1009,7 +899,7 @@ learn_cb (void *opaque, const void *buffer, size_t length)
   init_membuf (parm->data, 4096);
   return 0;
 }
-
+  
 /* Call the agent to learn about a smartcard */
 int
 gpgsm_agent_learn (ctrl_t ctrl)
@@ -1023,18 +913,14 @@ gpgsm_agent_learn (ctrl_t ctrl)
   if (rc)
     return rc;
 
-  rc = warn_version_mismatch (ctrl, agent_ctx, SCDAEMON_NAME, 2);
-  if (rc)
-    return rc;
-
   init_membuf (&data, 4096);
   learn_parm.error = 0;
   learn_parm.ctrl = ctrl;
   learn_parm.ctx = agent_ctx;
   learn_parm.data = &data;
   rc = assuan_transact (agent_ctx, "LEARN --send",
-                        learn_cb, &learn_parm,
-                        NULL, NULL,
+                        learn_cb, &learn_parm, 
+                        NULL, NULL, 
                         learn_status_cb, &learn_parm);
   xfree (get_membuf (&data, &len));
   if (rc)
@@ -1051,30 +937,29 @@ gpgsm_agent_passwd (ctrl_t ctrl, const char *hexkeygrip, const char *desc)
 {
   int rc;
   char line[ASSUAN_LINELENGTH];
-  struct default_inq_parm_s inq_parm;
 
   rc = start_agent (ctrl);
   if (rc)
     return rc;
-  inq_parm.ctrl = ctrl;
-  inq_parm.ctx = agent_ctx;
 
   if (!hexkeygrip || strlen (hexkeygrip) != 40)
     return gpg_error (GPG_ERR_INV_VALUE);
 
   if (desc)
     {
-      snprintf (line, DIM(line), "SETKEYDESC %s", desc);
+      snprintf (line, DIM(line)-1, "SETKEYDESC %s", desc);
+      line[DIM(line)-1] = 0;
       rc = assuan_transact (agent_ctx, line,
                             NULL, NULL, NULL, NULL, NULL, NULL);
       if (rc)
         return rc;
     }
 
-  snprintf (line, DIM(line), "PASSWD %s", hexkeygrip);
+  snprintf (line, DIM(line)-1, "PASSWD %s", hexkeygrip);
+  line[DIM(line)-1] = 0;
 
   rc = assuan_transact (agent_ctx, line, NULL, NULL,
-                        default_inq_cb, &inq_parm, NULL, NULL);
+                        default_inq_cb, ctrl, NULL, NULL);
   return rc;
 }
 
@@ -1087,18 +972,16 @@ gpgsm_agent_get_confirmation (ctrl_t ctrl, const char *desc)
 {
   int rc;
   char line[ASSUAN_LINELENGTH];
-  struct default_inq_parm_s inq_parm;
 
   rc = start_agent (ctrl);
   if (rc)
     return rc;
-  inq_parm.ctrl = ctrl;
-  inq_parm.ctx = agent_ctx;
 
-  snprintf (line, DIM(line), "GET_CONFIRMATION %s", desc);
+  snprintf (line, DIM(line)-1, "GET_CONFIRMATION %s", desc);
+  line[DIM(line)-1] = 0;
 
   rc = assuan_transact (agent_ctx, line, NULL, NULL,
-                        default_inq_cb, &inq_parm, NULL, NULL);
+                        default_inq_cb, ctrl, NULL, NULL);
   return rc;
 }
 
@@ -1126,9 +1009,9 @@ keyinfo_status_cb (void *opaque, const char *line)
   char **serialno = opaque;
   const char *s, *s2;
 
-  if ((s = has_leading_keyword (line, "KEYINFO")) && !*serialno)
+  if (!strncmp (line, "KEYINFO ", 8) && !*serialno)
     {
-      s = strchr (s, ' ');
+      s = strchr (line+8, ' ');
       if (s && s[1] == 'T' && s[2] == ' ' && s[3])
         {
           s += 3;
@@ -1166,7 +1049,8 @@ gpgsm_agent_keyinfo (ctrl_t ctrl, const char *hexkeygrip, char **r_serialno)
   if (!hexkeygrip || strlen (hexkeygrip) != 40)
     return gpg_error (GPG_ERR_INV_VALUE);
 
-  snprintf (line, DIM(line), "KEYINFO %s", hexkeygrip);
+  snprintf (line, DIM(line)-1, "KEYINFO %s", hexkeygrip);
+  line[DIM(line)-1] = 0;
 
   err = assuan_transact (agent_ctx, line, NULL, NULL, NULL, NULL,
                          keyinfo_status_cb, &serialno);
@@ -1183,195 +1067,3 @@ gpgsm_agent_keyinfo (ctrl_t ctrl, const char *hexkeygrip, char **r_serialno)
   return err;
 }
 
-
-
-/* Ask for the passphrase (this is used for pkcs#12 import/export.  On
-   success the caller needs to free the string stored at R_PASSPHRASE.
-   On error NULL will be stored at R_PASSPHRASE and an appropriate
-   error code returned.  If REPEAT is true the agent tries to get a
-   new passphrase (i.e. asks the user to confirm it).  */
-gpg_error_t
-gpgsm_agent_ask_passphrase (ctrl_t ctrl, const char *desc_msg, int repeat,
-                            char **r_passphrase)
-{
-  gpg_error_t err;
-  char line[ASSUAN_LINELENGTH];
-  char *arg4 = NULL;
-  membuf_t data;
-  struct default_inq_parm_s inq_parm;
-
-  *r_passphrase = NULL;
-
-  err = start_agent (ctrl);
-  if (err)
-    return err;
-  inq_parm.ctrl = ctrl;
-  inq_parm.ctx = agent_ctx;
-
-  if (desc_msg && *desc_msg && !(arg4 = percent_plus_escape (desc_msg)))
-    return gpg_error_from_syserror ();
-
-  snprintf (line, DIM(line), "GET_PASSPHRASE --data%s -- X X X %s",
-            repeat? " --repeat=1 --check --qualitybar":"",
-            arg4);
-  xfree (arg4);
-
-  init_membuf_secure (&data, 64);
-  err = assuan_transact (agent_ctx, line,
-                         put_membuf_cb, &data,
-                         default_inq_cb, &inq_parm, NULL, NULL);
-
-  if (err)
-    xfree (get_membuf (&data, NULL));
-  else
-    {
-      put_membuf (&data, "", 1);
-      *r_passphrase = get_membuf (&data, NULL);
-      if (!*r_passphrase)
-        err = gpg_error_from_syserror ();
-    }
-  return err;
-}
-
-
-
-/* Retrieve a key encryption key from the agent.  With FOREXPORT true
-   the key shall be use for export, with false for import.  On success
-   the new key is stored at R_KEY and its length at R_KEKLEN.  */
-gpg_error_t
-gpgsm_agent_keywrap_key (ctrl_t ctrl, int forexport,
-                         void **r_kek, size_t *r_keklen)
-{
-  gpg_error_t err;
-  membuf_t data;
-  size_t len;
-  unsigned char *buf;
-  char line[ASSUAN_LINELENGTH];
-  struct default_inq_parm_s inq_parm;
-
-  *r_kek = NULL;
-  err = start_agent (ctrl);
-  if (err)
-    return err;
-  inq_parm.ctrl = ctrl;
-  inq_parm.ctx = agent_ctx;
-
-  snprintf (line, DIM(line), "KEYWRAP_KEY %s",
-            forexport? "--export":"--import");
-
-  init_membuf_secure (&data, 64);
-  err = assuan_transact (agent_ctx, line,
-                         put_membuf_cb, &data,
-                         default_inq_cb, &inq_parm, NULL, NULL);
-  if (err)
-    {
-      xfree (get_membuf (&data, &len));
-      return err;
-    }
-  buf = get_membuf (&data, &len);
-  if (!buf)
-    return gpg_error_from_syserror ();
-  *r_kek = buf;
-  *r_keklen = len;
-  return 0;
-}
-
-
-
-
-/* Handle the inquiry for an IMPORT_KEY command.  */
-static gpg_error_t
-inq_import_key_parms (void *opaque, const char *line)
-{
-  struct import_key_parm_s *parm = opaque;
-  gpg_error_t err;
-
-  if (has_leading_keyword (line, "KEYDATA"))
-    {
-      assuan_begin_confidential (parm->ctx);
-      err = assuan_send_data (parm->ctx, parm->key, parm->keylen);
-      assuan_end_confidential (parm->ctx);
-    }
-  else
-    {
-      struct default_inq_parm_s inq_parm = { parm->ctrl, parm->ctx };
-      err = default_inq_cb (&inq_parm, line);
-    }
-
-  return err;
-}
-
-
-/* Call the agent to import a key into the agent.  */
-gpg_error_t
-gpgsm_agent_import_key (ctrl_t ctrl, const void *key, size_t keylen)
-{
-  gpg_error_t err;
-  struct import_key_parm_s parm;
-
-  err = start_agent (ctrl);
-  if (err)
-    return err;
-
-  parm.ctrl   = ctrl;
-  parm.ctx    = agent_ctx;
-  parm.key    = key;
-  parm.keylen = keylen;
-
-  err = assuan_transact (agent_ctx, "IMPORT_KEY",
-                         NULL, NULL, inq_import_key_parms, &parm, NULL, NULL);
-  return err;
-}
-
-
-
-/* Receive a secret key from the agent.  KEYGRIP is the hexified
-   keygrip, DESC a prompt to be displayed with the agent's passphrase
-   question (needs to be plus+percent escaped).  On success the key is
-   stored as a canonical S-expression at R_RESULT and R_RESULTLEN. */
-gpg_error_t
-gpgsm_agent_export_key (ctrl_t ctrl, const char *keygrip, const char *desc,
-                        unsigned char **r_result, size_t *r_resultlen)
-{
-  gpg_error_t err;
-  membuf_t data;
-  size_t len;
-  unsigned char *buf;
-  char line[ASSUAN_LINELENGTH];
-  struct default_inq_parm_s inq_parm;
-
-  *r_result = NULL;
-
-  err = start_agent (ctrl);
-  if (err)
-    return err;
-  inq_parm.ctrl = ctrl;
-  inq_parm.ctx = agent_ctx;
-
-  if (desc)
-    {
-      snprintf (line, DIM(line), "SETKEYDESC %s", desc);
-      err = assuan_transact (agent_ctx, line,
-                             NULL, NULL, NULL, NULL, NULL, NULL);
-      if (err)
-        return err;
-    }
-
-  snprintf (line, DIM(line), "EXPORT_KEY %s", keygrip);
-
-  init_membuf_secure (&data, 1024);
-  err = assuan_transact (agent_ctx, line,
-                         put_membuf_cb, &data,
-                         default_inq_cb, &inq_parm, NULL, NULL);
-  if (err)
-    {
-      xfree (get_membuf (&data, &len));
-      return err;
-    }
-  buf = get_membuf (&data, &len);
-  if (!buf)
-    return gpg_error_from_syserror ();
-  *r_result = buf;
-  *r_resultlen = len;
-  return 0;
-}
