@@ -1,6 +1,5 @@
 /* sign.c - Sign a message
- * Copyright (C) 2001, 2002, 2003, 2008,
- *               2010 Free Software Foundation, Inc.
+ *	Copyright (C) 2001, 2002, 2003, 2008 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -15,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -32,19 +31,19 @@
 #include <ksba.h>
 
 #include "keydb.h"
-#include "../common/i18n.h"
+#include "i18n.h"
 
 
 /* Hash the data and return if something was hashed.  Return -1 on error.  */
 static int
 hash_data (int fd, gcry_md_hd_t md)
 {
-  estream_t fp;
+  FILE *fp;
   char buffer[4096];
   int nread;
   int rc = 0;
 
-  fp = es_fdopen_nc (fd, "rb");
+  fp = fdopen ( dup (fd), "rb");
   if (!fp)
     {
       log_error ("fdopen(%d) failed: %s\n", fd, strerror (errno));
@@ -53,41 +52,40 @@ hash_data (int fd, gcry_md_hd_t md)
 
   do
     {
-      nread = es_fread (buffer, 1, DIM(buffer), fp);
+      nread = fread (buffer, 1, DIM(buffer), fp);
       gcry_md_write (md, buffer, nread);
     }
   while (nread);
-  if (es_ferror (fp))
+  if (ferror (fp))
     {
       log_error ("read error on fd %d: %s\n", fd, strerror (errno));
       rc = -1;
     }
-  es_fclose (fp);
+  fclose (fp);
   return rc;
 }
 
-
 static int
 hash_and_copy_data (int fd, gcry_md_hd_t md, ksba_writer_t writer)
 {
   gpg_error_t err;
-  estream_t fp;
+  FILE *fp;
   char buffer[4096];
   int nread;
   int rc = 0;
   int any = 0;
 
-  fp = es_fdopen_nc (fd, "rb");
+  fp = fdopen ( dup (fd), "rb");
   if (!fp)
     {
-      gpg_error_t tmperr = gpg_error_from_syserror ();
+      gpg_error_t tmperr = gpg_error (gpg_err_code_from_errno (errno));
       log_error ("fdopen(%d) failed: %s\n", fd, strerror (errno));
       return tmperr;
     }
 
   do
     {
-      nread = es_fread (buffer, 1, DIM(buffer), fp);
+      nread = fread (buffer, 1, DIM(buffer), fp);
       if (nread)
         {
           any = 1;
@@ -101,18 +99,18 @@ hash_and_copy_data (int fd, gcry_md_hd_t md, ksba_writer_t writer)
         }
     }
   while (nread && !rc);
-  if (es_ferror (fp))
+  if (ferror (fp))
     {
-      rc = gpg_error_from_syserror ();
+      rc = gpg_error (gpg_err_code_from_errno (errno));
       log_error ("read error on fd %d: %s\n", fd, strerror (errno));
     }
-  es_fclose (fp);
+  fclose (fp);
   if (!any)
     {
-      /* We can't allow signing an empty message because it does not
-         make much sense and more seriously, ksba_cms_build has
+      /* We can't allow to sign an empty message because it does not
+         make much sense and more seriously, ksba-cms_build has
          already written the tag for data and now expects an octet
-         string and an octet string of size 0 is illegal.  */
+         string but an octet string of zeize 0 is illegal. */
       log_error ("cannot sign an empty message\n");
       rc = gpg_error (GPG_ERR_NO_DATA);
     }
@@ -141,10 +139,10 @@ gpgsm_get_default_cert (ctrl_t ctrl, ksba_cert_t *r_cert)
   int rc;
   char *p;
 
-  hd = keydb_new ();
+  hd = keydb_new (0);
   if (!hd)
     return gpg_error (GPG_ERR_GENERAL);
-  rc = keydb_search_first (ctrl, hd);
+  rc = keydb_search_first (hd);
   if (rc)
     {
       keydb_release (hd);
@@ -180,7 +178,7 @@ gpgsm_get_default_cert (ctrl_t ctrl, ksba_cert_t *r_cert)
       ksba_cert_release (cert);
       cert = NULL;
     }
-  while (!(rc = keydb_search_next (ctrl, hd)));
+  while (!(rc = keydb_search_next (hd)));
   if (rc && rc != -1)
     log_error ("keydb_search_next failed: %s\n", gpg_strerror (rc));
 
@@ -211,18 +209,18 @@ get_default_signer (ctrl_t ctrl)
       return cert;
     }
 
-  rc = classify_user_id (opt.local_user, &desc, 0);
+  rc = keydb_classify_name (opt.local_user, &desc);
   if (rc)
     {
       log_error ("failed to find default signer: %s\n", gpg_strerror (rc));
       return NULL;
     }
 
-  kh = keydb_new ();
+  kh = keydb_new (0);
   if (!kh)
     return NULL;
 
-  rc = keydb_search (ctrl, kh, &desc, 1);
+  rc = keydb_search (kh, &desc, 1);
   if (rc)
     {
       log_debug ("failed to find default certificate: rc=%d\n", rc);
@@ -312,11 +310,11 @@ add_certificate_list (ctrl_t ctrl, ksba_cms_t cms, ksba_cert_t cert)
    be used if the value of this argument is NULL. */
 int
 gpgsm_sign (ctrl_t ctrl, certlist_t signerlist,
-            int data_fd, int detached, estream_t out_fp)
+            int data_fd, int detached, FILE *out_fp)
 {
   int i, rc;
   gpg_error_t err;
-  gnupg_ksba_io_t b64writer = NULL;
+  Base64Context b64writer = NULL;
   ksba_writer_t writer;
   ksba_cms_t cms = NULL;
   ksba_stop_reason_t stopreason;
@@ -331,7 +329,7 @@ gpgsm_sign (ctrl_t ctrl, certlist_t signerlist,
 
   audit_set_type (ctrl->audit, AUDIT_TYPE_SIGN);
 
-  kh = keydb_new ();
+  kh = keydb_new (0);
   if (!kh)
     {
       log_error (_("failed to allocate keyDB handle\n"));
@@ -340,10 +338,7 @@ gpgsm_sign (ctrl_t ctrl, certlist_t signerlist,
     }
 
   ctrl->pem_name = "SIGNED MESSAGE";
-  rc = gnupg_ksba_create_writer
-    (&b64writer, ((ctrl->create_pem? GNUPG_KSBA_IO_PEM : 0)
-                  | (ctrl->create_base64? GNUPG_KSBA_IO_BASE64 : 0)),
-     ctrl->pem_name, out_fp, &writer);
+  rc = gpgsm_create_writer (&b64writer, ctrl, out_fp, NULL, &writer);
   if (rc)
     {
       log_error ("can't create writer: %s\n", gpg_strerror (rc));
@@ -391,7 +386,7 @@ gpgsm_sign (ctrl_t ctrl, certlist_t signerlist,
           goto leave;
         }
 
-      /* Although we don't check for ambiguous specification we will
+      /* Although we don't check for ambigious specification we will
          check that the signer's certificate is usable and valid.  */
       rc = gpgsm_cert_use_sign_p (cert);
       if (!rc)
@@ -506,31 +501,28 @@ gpgsm_sign (ctrl_t ctrl, certlist_t signerlist,
   /* Check whether one of the certificates is qualified.  Note that we
      already validated the certificate and thus the user data stored
      flag must be available. */
-  if (!opt.no_chain_validation)
+  for (cl=signerlist; cl; cl = cl->next)
     {
-      for (cl=signerlist; cl; cl = cl->next)
-        {
-          size_t buflen;
-          char buffer[1];
+      size_t buflen;
+      char buffer[1];
 
-          err = ksba_cert_get_user_data (cl->cert, "is_qualified",
-                                         &buffer, sizeof (buffer), &buflen);
-          if (err || !buflen)
-            {
-              log_error (_("checking for qualified certificate failed: %s\n"),
-                         gpg_strerror (err));
-              rc = err;
-              goto leave;
-            }
-          if (*buffer)
-            err = gpgsm_qualified_consent (ctrl, cl->cert);
-          else
-            err = gpgsm_not_qualified_warning (ctrl, cl->cert);
-          if (err)
-            {
-              rc = err;
-              goto leave;
-            }
+      err = ksba_cert_get_user_data (cl->cert, "is_qualified",
+                                     &buffer, sizeof (buffer), &buflen);
+      if (err || !buflen)
+        {
+          log_error (_("checking for qualified certificate failed: %s\n"),
+                     gpg_strerror (err));
+          rc = err;
+          goto leave;
+        }
+      if (*buffer)
+        err = gpgsm_qualified_consent (ctrl, cl->cert);
+      else
+        err = gpgsm_not_qualified_warning (ctrl, cl->cert);
+      if (err)
+        {
+          rc = err;
+          goto leave;
         }
     }
 
@@ -550,7 +542,7 @@ gpgsm_sign (ctrl_t ctrl, certlist_t signerlist,
       algo = gcry_md_map_name (algoid);
       if (!algo)
         {
-          log_error ("unknown hash algorithm '%s'\n", algoid? algoid:"?");
+          log_error ("unknown hash algorithm `%s'\n", algoid? algoid:"?");
           rc = gpg_error (GPG_ERR_BUG);
           goto leave;
         }
@@ -763,7 +755,7 @@ gpgsm_sign (ctrl_t ctrl, certlist_t signerlist,
     }
   while (stopreason != KSBA_SR_READY);
 
-  rc = gnupg_ksba_finish_writer (b64writer);
+  rc = gpgsm_finish_writer (b64writer);
   if (rc)
     {
       log_error ("write failed: %s\n", gpg_strerror (rc));
@@ -781,7 +773,7 @@ gpgsm_sign (ctrl_t ctrl, certlist_t signerlist,
   if (release_signerlist)
     gpgsm_release_certlist (signerlist);
   ksba_cms_release (cms);
-  gnupg_ksba_destroy_writer (b64writer);
+  gpgsm_destroy_writer (b64writer);
   keydb_release (kh);
   gcry_md_close (data_md);
   return rc;
