@@ -23,11 +23,10 @@
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
+#include <fcntl.h>
 
 #include "i18n.h"
 #include "gpgtar.h"
-#include "../common/exectool.h"
-#include "../common/ccparray.h"
 
 
 
@@ -65,7 +64,7 @@ parse_xoctal (const void *data, size_t length, const char *filename)
             }
           else
             {
-              log_error ("%s: invalid octal number encountered - assuming 0\n",
+              log_error ("%s: invalid octal number encountered - assuming 0\n", 
                          filename);
               value = 0;
               break;
@@ -84,7 +83,7 @@ parse_header (const void *record, const char *filename)
   tar_header_t header;
   int use_prefix;
 
-  use_prefix = (!memcmp (raw->magic, "ustar", 5)
+  use_prefix = (!memcmp (raw->magic, "ustar", 5) 
                 && (raw->magic[5] == ' ' || !raw->magic[5]));
 
 
@@ -136,7 +135,7 @@ parse_header (const void *record, const char *filename)
     n = 0;
   memcpy (header->name+n, raw->name, namelen);
   header->name[n+namelen] = 0;
-
+  
   header->mode  = parse_xoctal (raw->mode, sizeof raw->mode, filename);
   header->uid   = parse_xoctal (raw->uid, sizeof raw->uid, filename);
   header->gid   = parse_xoctal (raw->gid, sizeof raw->gid, filename);
@@ -162,7 +161,7 @@ parse_header (const void *record, const char *filename)
     header->nrecords = (header->size + RECORDSIZE-1)/RECORDSIZE;
   else
     header->nrecords = 0;
-
+  
 
   return header;
 }
@@ -170,11 +169,10 @@ parse_header (const void *record, const char *filename)
 
 
 /* Read the next block, assming it is a tar header.  Returns a header
-   object on success in R_HEADER, or an error.  If the stream is
-   consumed, R_HEADER is set to NULL.  In case of an error an error
+   object on success or NULL one error.  In case of an error an error
    message has been printed.  */
-static gpg_error_t
-read_header (estream_t stream, tar_header_t *r_header)
+static tar_header_t
+read_header (estream_t stream)
 {
   gpg_error_t err;
   char record[RECORDSIZE];
@@ -182,7 +180,7 @@ read_header (estream_t stream, tar_header_t *r_header)
 
   err = read_record (stream, record);
   if (err)
-    return err;
+    return NULL;
 
   for (i=0; i < RECORDSIZE && !record[i]; i++)
     ;
@@ -192,8 +190,8 @@ read_header (estream_t stream, tar_header_t *r_header)
          end of archive mark.  */
       err = read_record (stream, record);
       if (err)
-        return err;
-
+        return NULL;
+      
       for (i=0; i < RECORDSIZE && !record[i]; i++)
         ;
       if (i != RECORDSIZE)
@@ -202,13 +200,11 @@ read_header (estream_t stream, tar_header_t *r_header)
       else
         {
           /* End of archive - FIXME: we might want to check for garbage.  */
-          *r_header = NULL;
-          return 0;
+          return NULL;
         }
     }
 
-  *r_header = parse_header (record, es_fname_get (stream));
-  return *r_header ? 0 : gpg_error_from_syserror ();
+  return parse_header (record, es_fname_get (stream));
 }
 
 
@@ -271,13 +267,12 @@ print_header (tar_header_t header, estream_t out)
 
 /* List the tarball FILENAME or, if FILENAME is NULL, the tarball read
    from stdin.  */
-gpg_error_t
-gpgtar_list (const char *filename, int decrypt)
+void
+gpgtar_list (const char *filename)
 {
   gpg_error_t err;
   estream_t stream;
-  estream_t cipher_stream = NULL;
-  tar_header_t header = NULL;
+  tar_header_t header;
 
   if (filename)
     {
@@ -288,63 +283,26 @@ gpgtar_list (const char *filename, int decrypt)
       if (!stream)
         {
           err = gpg_error_from_syserror ();
-          log_error ("error opening '%s': %s\n", filename, gpg_strerror (err));
-          return err;
+          log_error ("error opening `%s': %s\n", filename, gpg_strerror (err));
+          return;
         }
     }
   else
     stream = es_stdin;
 
+#ifdef HAVE_DOSISH_SYSTEM
   if (stream == es_stdin)
-    es_set_binary (es_stdin);
-
-  if (decrypt)
-    {
-      strlist_t arg;
-      ccparray_t ccp;
-      const char **argv;
-
-      cipher_stream = stream;
-      stream = es_fopenmem (0, "rwb");
-      if (! stream)
-        {
-          err = gpg_error_from_syserror ();
-          goto leave;
-        }
-
-      ccparray_init (&ccp, 0);
-
-      ccparray_put (&ccp, "--decrypt");
-      for (arg = opt.gpg_arguments; arg; arg = arg->next)
-        ccparray_put (&ccp, arg->d);
-
-      ccparray_put (&ccp, NULL);
-      argv = ccparray_get (&ccp, NULL);
-      if (!argv)
-        {
-          err = gpg_error_from_syserror ();
-          goto leave;
-        }
-
-      err = gnupg_exec_tool_stream (opt.gpg_program, argv,
-                                    cipher_stream, NULL, stream, NULL, NULL);
-      xfree (argv);
-      if (err)
-        goto leave;
-
-      err = es_fseek (stream, 0, SEEK_SET);
-      if (err)
-        goto leave;
-    }
+    setmode (es_fileno (es_stdin), O_BINARY);
+#endif
 
   for (;;)
     {
-      err = read_header (stream, &header);
-      if (err || header == NULL)
+      header = read_header (stream);
+      if (!header)
         goto leave;
-
+      
       print_header (header, es_stdout);
-
+      
       if (skip_data (stream, header))
         goto leave;
       xfree (header);
@@ -356,15 +314,14 @@ gpgtar_list (const char *filename, int decrypt)
   xfree (header);
   if (stream != es_stdin)
     es_fclose (stream);
-  if (stream != cipher_stream)
-    es_fclose (cipher_stream);
-  return err;
+  return;
 }
 
-gpg_error_t
-gpgtar_read_header (estream_t stream, tar_header_t *r_header)
+tar_header_t
+gpgtar_read_header (estream_t stream)
 {
-  return read_header (stream, r_header);
+  /*FIXME: Change to return an error code.  */
+  return read_header (stream);
 }
 
 void
